///////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////

#include "boardinfo-probe.h"
#include "qpi2c.h"
#include "cadv7604.h"
#include "hci.h"

#include "plgpio.h"
#include "udelay.h"

static Board_input_info board_input_info[] ={
[PL330_BOARD_NONE] = {
	.inputs_count = 0, //0
	.inputs_name= {
          "NONE",
         },
    },
[PL330_BOARD_C353] = {
	.inputs_count = 2,
	.inputs_name= {
          "HDMI",
          "VGA",
         },
	},	

[PL330_BOARD_CL334HN] = {
		.inputs_count = 1,
	    .inputs_name= {
          "HDMI",
         }, 
	},	

[PL330_BOARD_CM313B] = {
		.inputs_count = 1,
	    .inputs_name= {
          "SDI",
         },	
	},	
};

static void reset_ti3101(IHCIAPI *probe_hci)
{
	write_gpio(probe_hci,TI3101_RESET, 0);
	udelay_loop(5);
	write_gpio(probe_hci,TI3101_RESET, 1);
	udelay_loop(5);
}
static void hdmi_hotplug_cl334(IHCIAPI *probe_hci)
{
	write_gpio(probe_hci,ADV7604_HOTPLUG_CL334, 0);
	udelay_loop(10);
	write_gpio(probe_hci,ADV7604_HOTPLUG_CL334, 1);
	udelay_loop(100);
	write_gpio(probe_hci,ADV7604_HOTPLUG_CL334, 0);
	udelay_loop(10);	
};
static void hdmi_hotplug_c353(IHCIAPI *probe_hci)
{
	write_gpio(probe_hci,ADV7604_HOTPLUG_C353, 0);
	udelay_loop(10);
	write_gpio(probe_hci,ADV7604_HOTPLUG_C353, 1);
	udelay_loop(100);
	write_gpio(probe_hci,ADV7604_HOTPLUG_C353, 0);
	udelay_loop(10);	
};
static void reset_cadv7604(IHCIAPI *probe_hci)
{
	write_gpio(probe_hci, ADV7604_RESET, 0);
	udelay_loop(5);
	write_gpio(probe_hci, ADV7604_RESET, 1);
	udelay_loop(5);
	write_gpio(probe_hci, ADV7604_RESET, 0);
	udelay_loop(5);
	write_gpio(probe_hci, ADV7604_RESET, 1);  
}
	
static void probe_video_reset(IHCIAPI *probe_hci,Board_Types gpio_set)
{
	if ((gpio_set == PL330_BOARD_C353) || (gpio_set == PL330_BOARD_CL334HN))
	    reset_cadv7604(probe_hci);
	    
};

static void probe_hdmi_hotplug(IHCIAPI *probe_hci,Board_Types gpio_set)
{
	if (gpio_set == PL330_BOARD_CL334HN) 
	    hdmi_hotplug_cl334(probe_hci);
	if (gpio_set == PL330_BOARD_C353) 
	    hdmi_hotplug_c353(probe_hci);
	
};
static void probe_audio_reset(IHCIAPI *probe_hci,Board_Types gpio_set)
{
	if (gpio_set == PL330_BOARD_C353)
	    reset_ti3101(probe_hci);
	
};


static cadv7604_ops_t probe_7604_ops={
	.cadv7604_reset = &probe_video_reset,
    .cadv7604_hotplug = &probe_hdmi_hotplug,
    .cadv7604_TI3101_reset = &probe_audio_reset,
};

static cadv7604_device_cfg CL334HN_ADV7604_cfg =
{.internal_edid = true,PL330_BOARD_CL334HN,&board_input_info[PL330_BOARD_CL334HN],&probe_7604_ops};

static cadv7604_device_cfg C353_ADV7604_cfg =
{.internal_edid = false,PL330_BOARD_C353,&board_input_info[PL330_BOARD_C353],&probe_7604_ops};

PL330_boards_info frontend_boards_info[] = {
[PL330_BOARD_NONE] = {
	.boardname = "NONE",
    .device_cfg = {{
		"NONE",
		CHIP_UNKNOWN_FLAG,
		0xFF,
		NONE_BUS,
	}}, 

    },
[PL330_BOARD_C353] = {
    .boardname = "C353",
    .device_cfg = {{
		"ADV7604",
		VIDEO_7604,
		ADV7604_DEVICE_ADDR,
		I2C_BUS,
        &C353_ADV7604_cfg,
	},
	{
		"TI3101",
		AUDIO_TI3101,
		TI3101_DEVICE_ADDR,
		I2C_BUS,
        NULL,
	},{
        NULL,0,0,0,NULL
    }},
	},	


[PL330_BOARD_CL334HN] = {
		.boardname = "CL334HN",
        .device_cfg = {{
		"ADV7604",
		VIDEO_7604,
		ADV7604_DEVICE_ADDR,
		I2C_BUS,
        &CL334HN_ADV7604_cfg,
	},{
        NULL,0,0,0,NULL
    }},
	},	

[PL330_BOARD_CM313B] = {
		.boardname = "CM313B",
        .device_cfg = {{
		"CGV7601",
		VIDEO_7601,
		0xFF,
		SPI_BUS,
        NULL,
	},{
        NULL,0,0,0,NULL
    }},
	},	
};

static Chips_ProbeInfo PL330_chips_probe[] = {
	    {
			"ADV7604",
			ADV7604_DEVICE_ADDR,
	        ADV7604_SUB_ADDR,
	        ADV7604_PROBE_VALUE,
	        CHIP_ADV7604_FLAG,
	        
	        0xFF, 
	        I2C_BUS,
		},
		{
			"TI3101",
			TI3101_DEVICE_ADDR,
	        TI3101_SUB_ADDR,
	        TI3101_PROBE_VALUE,
	        CHIP_TI3101_FLAG,
	        
	        0xFF, 
	        I2C_BUS,
		},
		{
		     NULL,   
		}, 		
};

static int probe_device(II2C *This, u8 device_addrs, u8 reg)
{
	TQP_UINT8 value;
	EQP_Errors err = PROBE_FIAL;
	
    err = II2C_WriteThenRead(This,
							   device_addrs,	    //device address
							   1,					//write size
							   &reg,				//write buffer
							   1,					//read size
							   &value				//read buffer
							   );

	printk("::%s probe I2caddr %x status %d, value %x\n", __func__,(device_addrs*2),err,value);
							   
	return err;
}

PROBE_STATUS probe_frontend(II2C *This,PL330_boards_info * pl330_board_info)
{
	Chips_ProbeInfo *probe_PL330_devices = PL330_chips_probe;
    PROBE_STATUS probe_status = PROBE_SUCCESS;
	int i;
	int deviceaddr,subaddr,rvalue;
	int chip_flags=0;
	int board_mask=0;
    
    Board_Types  board_type = PL330_BOARD_NONE;
    
    
    if(!pl330_board_info)
    {
        printk("pl330_board_info == NULL ");
        
        probe_status = PROBE_FIAL;
        goto fail;
    }
	
	for (i=0; i < MAX_CHIP_PROBE; i++)
	{
		if (probe_PL330_devices[i].probe_bus == I2C_BUS)
		
		{ 
			deviceaddr = probe_PL330_devices[i].I2C_address;
			subaddr = probe_PL330_devices[i].I2C_subaddr;
			rvalue = probe_PL330_devices[i].I2C_value;
			// probe with I2C
			if(!probe_device(This,deviceaddr,subaddr))
			{
				chip_flags = probe_PL330_devices[i].chip_flag;
				printk("::an address detected 0x%x\n",deviceaddr*2);
			}
			else
			{
				printk("::can't detect address 0x%x\n",deviceaddr*2);
			}		 		
		}
		board_mask |= chip_flags;
		chip_flags =0;
	}

	
    switch(board_mask){

        case BOARD_CL334HN_MASK :
	    {
            board_type = PL330_BOARD_CL334HN;
            break;
	    }
	    case BOARD_C353_MASK :
	    {
		    board_type = PL330_BOARD_C353;
            break;
	    }
	    case BOARD_CM313B_MASK:
	    {
		    board_type = PL330_BOARD_CM313B;
  		break;
	    }
	    default:
	    {
		    board_type = PL330_BOARD_CM313B;	
		break;
	    }
    } 
    
    memcpy(pl330_board_info,&frontend_boards_info[board_type],sizeof(PL330_boards_info));
    
fail:

    return probe_status;
}

