#ifndef __PROBE_H__
#define __PROBE_H__

#include "qpi2c.h"
//#include "ci2c.h"


#define MAX_INPUT_NAME           9
#define MAX_INPUT_NUM            5

#define ADV7604_DEVICE_ADDR       (0x40 >>1)
#define ADV7604_SUB_ADDR          0xfb
#define ADV7604_PROBE_VALUE       0x68
#define ADV7604_HOTPLUG_CL334     _GPIO_6
#define ADV7604_HOTPLUG_C353      _GPIO_5
#define ADV7604_RESET             _GPIO_0
 
#define TI3101_DEVICE_ADDR        (0x30 >>1)
#define TI3101_SUB_ADDR           0x00
#define TI3101_PROBE_VALUE        0x00
#define TI3101_RESET              _GPIO_1  

#define BOARD_NAME_SIZE 64

#define MAX_CHIP_PROBE            3 //probe target 1/2/3

/**********************************************************/
// Define board enum for constructor usage
/**********************************************************/
typedef enum
{
    PROBE_SUCCESS=0,
    PROBE_FIAL=-1,
}PROBE_STATUS;

typedef enum  //probe board result
{
	PL330_BOARD_NONE = 0,
	PL330_BOARD_C353 = 1,
	PL330_BOARD_CL334HN =2,
	PL330_BOARD_CM313B =3,
	PL330_BOARD_COUNT,
}Board_Types;

typedef enum _Peripherals_Type{
    VIDEO_7601,
    VIDEO_7604,
    AUDIO_TI3101,
    PERIPHERALS_MAX,
}Peripherals_Type;

enum Peripherals_Class{
    _VIDEO_CLASS,
    _AUDIO_CLASS,
    _MISC_CLASS,
};
typedef enum {
    _GPIO_0 = 0,
    _GPIO_1 = 1,
    _GPIO_5 = 5,
    _GPIO_6 = 6,
    _GPIO_unuse = 99,
}Peripherals_Gpio;

#define PERIPHERALS_CLASS_SHIFT 30
#define VIDEO_CLASS _VIDEO_CLASS << PERIPHERALS_CLASS_SHIFT
#define AUDIO_CLASS _AUDIO_CLASS << PERIPHERALS_CLASS_SHIFT
#define MISC_CLASS  _MISC_CLASS  << PERIPHERALS_CLASS_SHIFT
typedef enum
{
	CHIP_ADV7604_FLAG = (1<<VIDEO_7604) |  VIDEO_CLASS, 
	CHIP_CGV7601_FLAG = (1<<VIDEO_7601) |  VIDEO_CLASS, 
	CHIP_TI3101_FLAG  = (1<<AUDIO_TI3101) | AUDIO_CLASS, 
	//CHIP_EEPROM_FLAG  = 0x0100,
	CHIP_UNKNOWN_FLAG = 0xFFFF,
}Chip_Flags;

/**************************************************************/
// Define parameters for probe function usage
/***************************************************************/

enum
{
	BOARD_C353_MASK = CHIP_ADV7604_FLAG | CHIP_TI3101_FLAG,
	BOARD_CL334HN_MASK  = CHIP_ADV7604_FLAG,
	BOARD_CM313B_MASK = CHIP_CGV7601_FLAG,
};

typedef enum
{
	I2C_BUS = 0,
	SPI_BUS,
	NONE_BUS,
	
}Chip_BusType;

typedef struct
{
	unsigned char          *name; // peripheral name
	int                    I2C_address; //I2C
	int                    I2C_subaddr;
	int                    I2C_value;
	Chip_Flags             chip_flag;
	
	int                    SPI_select;//SDI
	Chip_BusType           probe_bus;
}Chips_ProbeInfo;


/******************************************************************/
// Define board info for initial and frontend setting
/******************************************************************/
typedef struct
{
	unsigned int	   inputs_count; //board inputs
	const char         *inputs_name[MAX_INPUT_NUM];
}Board_input_info;

typedef struct
{
	unsigned char          *name; // peripheral name 
    Peripherals_Type       peripheral_type;
	int                    I2C_add_cfg; //I2C
	Chip_BusType           bus_type_cfg;
	void                   *device_set;
}Chip_Cfg;


typedef struct {
    unsigned char      boardname[BOARD_NAME_SIZE]; // device name
	Chip_Cfg           device_cfg[MAX_CHIP_PROBE];
}PL330_boards_info;

PROBE_STATUS probe_frontend(II2C *This,PL330_boards_info * pl330_board_info);
//void frontend_init(IFrontend *frontend,frontend_setup_t *setup_info);

#endif






