///////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2011 Sigma Designs, Inc.  All Rights Reserved.   
//
// Ccomponent.h
//
// Description: 
//
//  component declaration
// 
// History:
//	Dec 18, 2009	(dt) created
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////

#ifndef __CCOMPONENT_H__
#define __CCOMPONENT_H__

#include <linux/types.h>
#include <linux/file.h>
#include <linux/fs.h>
#include <linux/poll.h>
#include "pldevioctl.h"

#include "qptypes.h"
#include "cobject.h"
#include "cport.h"


/////////////////////////////////////////////////////////////////////////////
//
//


#ifndef __CDEVICE_FWD_DEFINED__
#define __CDEVICE_FWD_DEFINED__
typedef struct CDevice CDevice;
#endif //!__CDEVICE_FWD_DEFINED__


#ifndef __CCOMPONENT_FWD_DEFINED__
#define __CCOMPONENT_FWD_DEFINED__
typedef struct CComponent CComponent;
#endif //!__CCOMPONENT_FWD_DEFINED__


struct CComponent
{
	// public CObject
	//
	CObject				m_Object;

	// CComponent
	//
    struct video_device *m_pVideoDev;
	struct pl330_audio_dev *m_pAudioDev;
	wait_queue_head_t 	wait_frame;
	CDevice				*m_pDevice;
	COMPONENT_TYPE		m_type;
	COMPONENT_TYPE		m_dummy_type;
	TQP_OBJECT_HANDLE	m_hTask;	// codec lib task handle
	TQP_OBJECT_HANDLE	m_hObject;	// handle acquired from object manager
	CPort				m_Ports[MAX_COMPONENT_PORT];
        int                     out_width;
        int                     out_height;

};		


CComponent * CComponent_Constructor(CComponent *pComponent,
									struct video_device *pVideoDev,
									CDevice *pDevice,
									COMPONENT_TYPE type
									);
void CComponent_Destructor(CComponent *This);
EQP_Errors CComponent_Open(CComponent *This);
EQP_Errors CComponent_Close(CComponent *This);
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,29)
long CComponent_Ioctl(struct file *filp, 
					 unsigned int cmd, 
					 void *arg
					 );
#else // LINUX_VERSION_CODE < KERNEL_VERSION(2,6,29)
long CComponent_Ioctl(struct inode *inode, 
					 struct file *filp, 
					 unsigned int cmd, 
					 void *arg
					 );
#endif //LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,29)
int CComponent_Mmap(CComponent *This, 
					struct vm_area_struct *vma
					);
int CComponent_Poll(CComponent *This,
					struct file *filp, poll_table *wait
					);
// access functions
//
static inline TQP_OBJECT_HANDLE CComponent_GetTaskHandle(CComponent *This) {return (This->m_hTask);}


int ccomponent_port_open(CComponent *This, PCOMPONENT_PORT_OPEN pOpenParam);

int ccomponent_port_close(CComponent *This, COMPONENT_PORT_TYPE type);

int ccomponent_start_index(CComponent *This, COMPONENT_PORT_TYPE type);

int ccomponent_start(CComponent *This);

int ccomponent_stop_index(CComponent *This, COMPONENT_PORT_TYPE type);

int ccomponent_stop(CComponent *This);

int ccomponent_acquire_index(CComponent *This, COMPONENT_PORT_TYPE type);

int ccomponent_acquire(CComponent *This);


int snd_pl330_qbuf(CComponent *component, struct v4l2_buffer *buf);
int snd_pl330_dqbuf(CComponent *component, struct v4l2_buffer *buf);

#endif //__CCOMPONENT_H__




