///////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2011 Sigma Designs, Inc.   All Rights Reserved.
//
// Cdevice.c
//
// Description:
//
//  cdevice implementation
//
// History:
//	Jan 18, 2010	(dt) created
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////


#include "qlautoconf.h"

#include <linux/version.h>
#include <linux/module.h>
#include <linux/videodev2.h>

#include <linux/init.h>
#include <linux/delay.h>
#include <linux/sched.h>
#include <linux/fs.h>
#include <linux/interrupt.h>
#include <linux/spinlock.h>
//#include <linux/video_decoder.h>
//#include <linux/video_encoder.h>
#include <linux/i2c.h>
#include <linux/i2c-algo-bit.h>
#include <linux/list.h>
#include <linux/unistd.h>
//#include <linux/byteorder/swab.h>
#include <linux/pagemap.h>
#include <linux/ctype.h>
#include <linux/slab.h>
#include <linux/poll.h>
#include <linux/page-flags.h>
#include <asm/uaccess.h>
#if LINUX_VERSION_CODE > KERNEL_VERSION(3,3,0)
#include <asm/switch_to.h>
#else
#include <asm/system.h>
#endif
#include <asm/io.h>

#include <linux/vmalloc.h>

#if defined(CONFIG_HOST_PLATFORM_PL330SOC)
#include <mach/pl330xp.h>
#endif

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,26)
#include <linux/semaphore.h>
#else //LINUX_VERSION_CODE < KERNEL_VERSION(2,6,26)
#include <asm/semaphore.h>
#endif //LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,26)

#include "qpdbgmgr.h"
#include "pldevioctl.h"
#include "ccomponent.h"
#include "cdevice.h"
#include "qplinuxkermap.h"
#include "qlfops.h"

#include "pl330-dbg.h"
#include "pl330-version.h"

MODULE_AUTHOR(DRIVER_AUTHOR);
MODULE_VERSION(DRIVER_VERSION);
MODULE_DESCRIPTION(DRIVER_DESC);
MODULE_LICENSE("GPL");

extern struct semaphore g_module_lock;

/* pl330-alsa.c */
extern struct pl330_audio_dev *pl330_audio_register(
                                                    CDevice *dev);
extern struct pl330_audio_dev *pl330_audio_unregister(
                                                      CDevice *dev);

extern void ccomponent_wait_qbuf(struct work_struct *work);
extern void ccomponent_wait_dqbuf(struct work_struct *work);

#ifdef CONFIG_HOST_PLATFORM_ASPEN
#define PL300_PCIE_EVK_VID_FW_FILENAME "/tmp/bin/qpvidfwpcie.bin"
#define PL300_AUD_FW_FILENAME   "/tmp/bin/qpaudfw.bin"
#else
#define PL300_PCIE_EVK_VID_FW_FILENAME "/opt/qpvidfwpcie.bin"
#define PL300_AUD_FW_FILENAME   "/opt/qpaudfw.bin"
#endif

/////////////////////////////////////////////////////////////////////////////
//
//

CDevice *g_pPlV4lDevices[PLV4LDEV_MAX_DEVICES] ={
    0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0
};

/////////////////////////////////////////////////////////////////////////////
//
//

// V4L interfaces
//

static __attribute__((__unused__)) ssize_t plv4l_read(struct file *filp,
                                                      char *buf,
                                                      size_t count,
                                                      loff_t *pos
                                                      )
{
    return (-EPERM);
}

static __attribute__((__unused__)) ssize_t plv4l_write(struct file *filp,
                                                       const char *buf,
                                                       size_t count,
                                                       loff_t *pos
                                                       )
{
    return (-EPERM);
}

CComponent *snd_pl330_component_open(CDevice *This)
{
    CComponent *component;
    int err = 0;

    down(&g_module_lock);

    component = kmalloc(sizeof (CComponent), GFP_KERNEL);
    if (!component)
    {
        up(&g_module_lock);
        return NULL;
    }

    if (!CComponent_Constructor(component,
                                NULL,
                                This, COMPONENT_TYPE_PCM))
    {
        pl330_trace(PL330_TRACE_SND,
                    "%s() create audio component failed\n",
                    __func__);
        err = -EIO;
        goto done;
    }
    This->pcm_component = component;

    pl330_trace(PL330_TRACE_CALL, "%s "
                "audio component m_type = %d\n", __func__,
                This->pcm_component->m_type);

    /* alloc task */
    CComponent_Open(component);
done:
    up(&g_module_lock);

    return component;
}

void snd_pl330_component_close(CDevice *This, CComponent *component)
{
    down(&g_module_lock);

    pl330_trace(PL330_TRACE_CALL, "%s\n", __func__);

    CComponent_Close(component);
    CComponent_Destructor(component);
    kfree(component);
    This->pcm_component = NULL;

    //done:
    up(&g_module_lock);
}

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,29)
static int plv4l_open(struct file *filp)
#else //LINUX_VERSION_CODE < KERNEL_VERSION(2,6,29)

static int plv4l_open(struct inode *inode,
                      struct file *filp
                      )
#endif //LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,29)
{
    long lErr = 0;
    int i, j;
    CDevice *pDevice;
    CComponent *pComponent;
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,29)
    int minor = video_devdata(filp)->minor;
    ;
#else //LINUX_VERSION_CODE < KERNEL_VERSION(2,6,29)
    int minor = MINOR(inode->i_rdev);
#endif //LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,29)

    down(&g_module_lock);

    for (i = 0; i < PLV4LDEV_MAX_DEVICES; i++)
    {
        pDevice = g_pPlV4lDevices[i];
        if (pDevice)
        {
            for (j = 0; j < MAX_COMPONENT_TYPE; j++)
            {
                if (pDevice->m_pVideoDevices[j])
                {
                    if (pDevice->m_pVideoDevices[j]->minor == minor)
                    {
                        // got it
                        pl330_trace(PL330_TRACE_CALL, "%s minor %d\n", __func__, minor);
                        goto _do_open;
                    }
                }
            }
        }
    }

    up(&g_module_lock);


    // not found
    //
    QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
                 "%s() minor(%d) not found\n",
                 __FUNCTION__,
                 minor
                 ));
    return (-ENODEV);

_do_open:

    pl330_trace(PL330_TRACE_CALL,
                "%s create and open component type=%d\n",
                __func__, j);
    // create and open the component
    //
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,24)) && !defined(CONFIG_HOST_PLATFORM_PL330SOC)
    pComponent = kmalloc(sizeof (CComponent),
                         GFP_KERNEL
                         );
#else
    pComponent = vmalloc(sizeof (CComponent));
#endif
    if (pComponent)
    {
        if (!CComponent_Constructor(pComponent,
                                    pDevice->m_pVideoDevices[j],
                                    pDevice,
                                    j
                                    ))
        {
            QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
                         "%s() minor(%d) CComponent_Constructor failed!!!\n",
                         __FUNCTION__,
                         minor
                         ));
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,24)) && !defined(CONFIG_HOST_PLATFORM_PL330SOC)
            kfree((void *) pComponent);
#else
            vfree((void *) pComponent);
#endif
            lErr = -ENOMEM;
        }
        else
        {
            // component open will add the component to the object manager
            pDevice->m_pComponent = pComponent;
            CComponent_Open(pComponent);
        }
    }
    else
    {
        QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
                     "%s() minor(%d) can not allocate pComponent\n",
                     __FUNCTION__,
                     minor
                     ));
        up(&g_module_lock);
        return -ENOMEM;
    }

    //done:
    // save to file private data
    //
    filp->private_data = pComponent;
    up(&g_module_lock);

    return (lErr);
}


static long aver_usercopy(struct file *file, 
                             unsigned int cmd, 
                             unsigned long arg)
{
    char    sbuf[128];
    void    *mbuf = NULL;
    void    *parg = (void *)arg;
    long    err  = -EINVAL;

    /*  Copy arguments into temp kernel buffer  */
    if (_IOC_DIR(cmd) != _IOC_NONE) {
        if (_IOC_SIZE(cmd) <= sizeof(sbuf)) {
            parg = sbuf;
        } else {
            /* too big to allocate from stack */
            mbuf = kmalloc(_IOC_SIZE(cmd), GFP_KERNEL);
            if (NULL == mbuf)
                return -ENOMEM;
            parg = mbuf;
        }

        err = -EFAULT;
        if (_IOC_DIR(cmd) & _IOC_WRITE) {
            unsigned int n = _IOC_SIZE(cmd);

            if (copy_from_user(parg, (void __user *)arg, n))
                goto out;

            /* zero out anything we don't copy from userspace */
            if (n < _IOC_SIZE(cmd))
                memset((u8 *)parg + n, 0, _IOC_SIZE(cmd) - n);
        } else {
            /* read-only ioctl */
            memset(parg, 0, _IOC_SIZE(cmd));
        }
    }

    /* Handles IOCTL */
    err = CComponent_Ioctl(file, cmd, parg);
    if (err == -ENOIOCTLCMD)
        err = -ENOTTY;

    if (err < 0)
        goto out;

    /*  Copy results into user buffer  */
    switch (_IOC_DIR(cmd)) {
        case _IOC_READ:
        case (_IOC_WRITE | _IOC_READ):
            if (copy_to_user((void __user *)arg, parg, _IOC_SIZE(cmd)))
                err = -EFAULT;
            break;
    }

out:
    kfree(mbuf);
    return err;
}

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,29)
static long plv4l_ioctl(struct file *filp,
                        unsigned int cmd,
                        unsigned long arg
                        )
#else //LINUX_VERSION_CODE < KERNEL_VERSION(2,6,29)

static int plv4l_ioctl(struct inode *inode,
                       struct file *filp,
                       unsigned int cmd,
                       unsigned long arg
                       )
#endif //LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,29)
{
    int i, j;
    CDevice *pDevice;
    CComponent *pComponent = (CComponent *) filp->private_data;
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,29)
    int minor = video_devdata(filp)->minor;
    
#else //LINUX_VERSION_CODE < KERNEL_VERSION(2,6,29)
    int minor = MINOR(inode->i_rdev);
#endif //LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,29)

    
    down(&g_module_lock);

    for (i = 0; i < PLV4LDEV_MAX_DEVICES; i++)
    {
        pDevice = g_pPlV4lDevices[i];
        if (pDevice)
        {
            for (j = 0; j < MAX_COMPONENT_TYPE; j++)
            {
                if (pDevice->m_pVideoDevices[j])
                {
                    if (pDevice->m_pVideoDevices[j]->minor == minor)
                    {
                        // got it
                        up(&g_module_lock);
                        goto _do_ioctl;
                    }
                }
            }
        }
    }

    up(&g_module_lock);


    // not found
    //
    QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_WARN),
                 "%s() minor(%d) not found\n",
                 __FUNCTION__,
                 minor
                 ));
    return (-ENODEV);

_do_ioctl:
    pComponent->m_dummy_type = j;

#if LINUX_VERSION_CODE >= KERNEL_VERSION(4,15,0)
{
    long ret;
        ret=aver_usercopy(filp,
                           cmd,
                           arg/*,
                           CComponent_Ioctl*/
                           );
        
    return ret;
}
#else
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,29)
{
    long ret;
        ret=video_usercopy(filp,
                           cmd,
                           arg,
                           CComponent_Ioctl
                           );
        
    return ret;
}
#else //LINUX_VERSION_CODE < KERNEL_VERSION(2,6,29)
    return (video_usercopy(inode,
                           filp,
                           cmd,
                           arg,
                           CComponent_Ioctl
                           ));
#endif //LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,29)
#endif
}



#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,29)
static int plv4l_close(struct file *filp)
#else //LINUX_VERSION_CODE < KERNEL_VERSION(2,6,29)

static int plv4l_close(struct inode *inode,
                       struct file *filp
                       )
#endif //LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,29)
{
    long lErr = -EPERM;
    int i, j;
    CDevice *pDevice;
    CComponent *pComponent = (CComponent *) filp->private_data;
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,29)
    int minor = video_devdata(filp)->minor;
    ;
#else //LINUX_VERSION_CODE < KERNEL_VERSION(2,6,29)
    int minor = MINOR(inode->i_rdev);
#endif //LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,29)

    down(&g_module_lock);

    for (i = 0; i < PLV4LDEV_MAX_DEVICES; i++)
    {
        pDevice = g_pPlV4lDevices[i];
        if (pDevice)
        {
            for (j = 0; j < MAX_COMPONENT_TYPE; j++)
            {
                if (pDevice->m_pVideoDevices[j])
                {
                    if (pDevice->m_pVideoDevices[j]->minor == minor)
                    {
                        // got it
                        goto _do_close;
                    }
                }
            }
        }
    }

    up(&g_module_lock);

    // not found
    //
    QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
                 "%s() minor(%d) not found\n",
                 __FUNCTION__,
                 minor
                 ));
    return (-ENODEV);

_do_close:

    pl330_trace(PL330_TRACE_CALL, "%s close component type=%d\n",
                __func__, j);
    if (pComponent)
    {
        pl330_trace(PL330_TRACE_CALL, "%s release component type=%d\n",
                    __func__, j);
        CComponent_Close(pComponent);
        CComponent_Destructor(pComponent);
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,24)) && !defined(CONFIG_HOST_PLATFORM_PL330SOC)
        kfree((void *) pComponent);
#else
        vfree((void *) pComponent);
#endif
        pDevice->m_pComponent = NULL;
        lErr = 0;
    }
    else
    {
        pl330_trace(PL330_TRACE_CALL, "%s component type %d is null\n",
                    __func__, j);
    }
    //done:
    up(&g_module_lock);
    return lErr;
}

static unsigned int plv4l_poll(struct file *filp,
                               poll_table *wait
                               )
{
    long lErr = 0;
    CComponent *pComponent = (CComponent *) filp->private_data;

    //printk("plv4l_poll called\n");

    if (pComponent)
    {
        lErr = CComponent_Poll(pComponent,
                               filp, wait
                               );
    }

    return (lErr); /*POLLIN|POLLRDNORM*/
    //return (0);
}

static void plv4l_mmap_open(struct vm_area_struct *vma)
{
    return;
}

static void plv4l_mmap_close(struct vm_area_struct *vma)
{
    return;
}



struct vm_operations_struct plv4l_mmap_ops ={
    open : plv4l_mmap_open,
    close : plv4l_mmap_close,
};

static int plv4l_mmap(struct file *filp,
                      struct vm_area_struct *vma
                      )
{
    long lErr = -EPERM;
    CComponent *pComponent = (CComponent *) filp->private_data;

    vma->vm_ops = &plv4l_mmap_ops;

    if (pComponent)
    {
        return (CComponent_Mmap(pComponent,
                                vma
                                ));
    }

    return lErr;
}



#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,29)
static struct v4l2_file_operations plv4l_fops =
#else //LINUX_VERSION_CODE < KERNEL_VERSION(2,6,29)
static struct file_operations plv4l_fops =
#endif //LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,29)
{
    owner : THIS_MODULE,
    open : plv4l_open,
    unlocked_ioctl : plv4l_ioctl,
    release : plv4l_close,
    poll : plv4l_poll,
    mmap : plv4l_mmap,
};


/* ------------------------------------------------------------------
   IOCTL vidioc handling
   ------------------------------------------------------------------*/
static int vidioc_querycap(struct file *file, void  *priv,
        struct v4l2_capability *cap)
{

    return 0;
}

static int vidioc_enum_fmt_vid_cap(struct file *file, void  *priv,
        struct v4l2_fmtdesc *f)
{
    return 0;
}

static int vidioc_g_fmt_vid_cap(struct file *file, void *priv,
        struct v4l2_format *f)
{
    return 0;
}

static int vidioc_try_fmt_vid_cap(struct file *file, void *priv,
        struct v4l2_format *f)
{
    return 0;
}

static int vidioc_s_fmt_vid_cap(struct file *file, void *priv,
        struct v4l2_format *f)
{
    return 0;
}

static int vidioc_reqbufs(struct file *file, void *priv,
        struct v4l2_requestbuffers *p)
{
    return 0;
}

/*
 * user get the video buffer status
 * for mmap, get the map offset
 * */
static int vidioc_querybuf(struct file *file, void *priv, struct v4l2_buffer *p)
{
    return 0;
}

static int vidioc_qbuf(struct file *file, void *priv, struct v4l2_buffer *p)
{

    return 0;
}

/*
 * check if have available frame, if have no frame return <0;
 * if block, it will wait until there is frame available, (vb->done event).
 * if have frame, return 0 and transfer the buffer info to user.
 * */
static int vidioc_dqbuf(struct file *file, void *priv, struct v4l2_buffer *p)
{
    return 0;
}

static int vidioc_enum_input(struct file *file, void *priv,
        struct v4l2_input *inp)
{
    return 0;
}

static int vidioc_g_input(struct file *file, void *priv, unsigned int *i)
{
    return 0;
}

static int vidioc_s_input(struct file *file, void *priv, unsigned int i)
{
    return (0);
}

static int vidioc_g_parm(struct file *file, void *__fh,
        struct v4l2_streamparm *sp)
{
    return 0;
}

static int vidioc_s_parm(struct file *file, void *__fh,
        struct v4l2_streamparm *sp)
{
    return 0;
}

static int vidioc_streamon(struct file *file, void *priv, enum v4l2_buf_type i)
{
    return 0;
}

static int vidioc_streamoff(struct file *file, void *priv, enum v4l2_buf_type i)
{
    return 0;
}

/* --- controls ---------------------------------------------- */
static int vidioc_queryctrl(struct file *file, void *priv,
        struct v4l2_queryctrl *qc)
{
    return 0;
}

static int vidioc_g_ctrl(struct file *file, void *priv,
        struct v4l2_control *ctrl)
{
    return 0;
}

static int vidioc_s_ctrl(struct file *file, void *priv,
        struct v4l2_control *ctrl)
{

    return 0;
}


static int vidioc_enum_framesizes(struct file *file, void *priv,
        struct v4l2_frmsizeenum *fsize)
{

    return 0;
}

extern unsigned int enum_frameinterval_min;
static int vidioc_enum_frameintervals(struct file *file, void *fh,
                   struct v4l2_frmivalenum *fival)
{
    return 0;
}

#if (LINUX_VERSION_CODE < KERNEL_VERSION(5,0,0))
static int vidioc_cropcap(struct file *file, void *priv,
        struct v4l2_cropcap *ccap)
{

    return 0;
}

static int vidioc_g_crop(struct file *file, void *priv,
        struct v4l2_crop *c)
{
    return 0;
}

#if (LINUX_VERSION_CODE < KERNEL_VERSION(3,7,0))
static int vidioc_s_crop(struct file *file, void *priv,
        struct v4l2_crop *c)
#else
static int vidioc_s_crop(struct file *file, void *priv,
        const struct v4l2_crop *c)
#endif
{

    return 0;
}
#endif

static const struct v4l2_ioctl_ops plv4l_ioctl_ops = {
    .vidioc_querycap          = vidioc_querycap,
    .vidioc_enum_fmt_vid_cap  = vidioc_enum_fmt_vid_cap,
    .vidioc_g_fmt_vid_cap     = vidioc_g_fmt_vid_cap,
    .vidioc_try_fmt_vid_cap   = vidioc_try_fmt_vid_cap,
    .vidioc_s_fmt_vid_cap     = vidioc_s_fmt_vid_cap,
    .vidioc_reqbufs           = vidioc_reqbufs,
    .vidioc_querybuf          = vidioc_querybuf,
    .vidioc_qbuf              = vidioc_qbuf,
    .vidioc_dqbuf             = vidioc_dqbuf,
    .vidioc_enum_input        = vidioc_enum_input,
    .vidioc_g_input           = vidioc_g_input,
    .vidioc_s_input           = vidioc_s_input,
    .vidioc_g_parm            = vidioc_g_parm,
    .vidioc_s_parm            = vidioc_s_parm,
    .vidioc_streamon          = vidioc_streamon,
    .vidioc_streamoff         = vidioc_streamoff,
    .vidioc_queryctrl         = vidioc_queryctrl,
    .vidioc_g_ctrl            = vidioc_g_ctrl,
    .vidioc_s_ctrl            = vidioc_s_ctrl,
    .vidioc_enum_framesizes   = vidioc_enum_framesizes,
    .vidioc_enum_frameintervals
                              = vidioc_enum_frameintervals,

#if (LINUX_VERSION_CODE < KERNEL_VERSION(5,0,0))
    .vidioc_cropcap      	  = vidioc_cropcap,
    .vidioc_s_crop       	  = vidioc_s_crop,
    .vidioc_g_crop       	  = vidioc_g_crop,
#endif
};

static struct video_device plv4l_videodevice ={
    .name = "SDL H.264 Codec (V4L)",
    .device_caps = V4L2_CAP_VIDEO_CAPTURE | V4L2_CAP_STREAMING,
    .fops = &plv4l_fops,
    // v4l2 ioctl ops
    //.ioctl_ops = &plv4l_ioctl_ops,
    .release = video_device_release,
    /* codec's v4l2_ctrl_handler
    .ctrl_handler = 
     */
};



/////////////////////////////////////////////////////////////////////////////
//
//


#ifdef __LINUX24__

inline struct video_device *video_device_alloc(void)
{
    struct video_device *vfd;

    vfd = kmalloc(sizeof (*vfd), GFP_KERNEL);
    if (NULL == vfd)
    {
        return NULL;
    }
    memset(vfd, 0, sizeof (*vfd));
    return vfd;
}
#endif //__LINUX24__

inline void video_device_release(struct video_device *vfd)
{
    if (vfd) kfree(vfd);
}

static EQP_Errors CDevice_Callback(TQP_PVOID Context,
                                   TQP_UINT32 dwCode,
                                   TQP_PVOID pParam
                                   )
{
    return (QPERR_SUCCESS);
}


/////////////////////////////////////////////////////////////////////////////
//
//


// constructor
//

CDevice * CDevice_Constructor(CDevice *pDevice,
                              TQP_UINT32 id,
                              PLV4LDEV_TYPE type,
                              PPLV4LDEV_PARENT_DEV pDev,
                              struct device *pGenericDevice,
                              TQP_PVOID RegisterBase,
                              TQP_PVOID MemoryBase,
                              TQP_UINT32 PageSize,
                              TQP_UINT32 InterruptNumber
                              )
{
    if (pDevice)
    {
        char *pvidfilename = 0;
        char *paudfilename = 0;
        int i;

        // CObject
        //
        CObject_Constructor(&pDevice->m_Object,
                            QP_NULL,
                            OBJECT_CRITICAL_LIGHT
                            );

        // initialize members
        //
        pDevice->m_id = id;
        if ((type > MAX_PLV4LDEV_TYPE) ||
                (type <= PLV4LDEV_TYPE_NONE)
                )
        {
            return (QP_NULL);
        }
        pDevice->m_type = type;

        for (i = 0; i < MAX_COMPONENT_TYPE; i++)
        {
            pDevice->m_pVideoDevices[i] = QP_NULL;
        }

        pDevice->m_device.read = pDev->read;
        pDevice->m_pGenericDevice = pGenericDevice;

        pl330_trace(PL330_TRACE_CALL, "%s device type %d\n", __func__, type);

        switch (type)
        {
        case PLV4LDEV_TYPE_QL300_PCIE_EVK:
            pDevice->m_initData.bDownloadFW = QP_TRUE;

            pDevice->m_initData.mpgCodecInitData.bDontInitHW = QP_FALSE;
#ifdef CONFIG_FW_INCLUDED_IN_DRIVER
            pDevice->m_initData.mpgCodecInitData.bUseExtFW = QP_FALSE;
#else
            pDevice->m_initData.mpgCodecInitData.bUseExtFW = QP_TRUE;
#endif
            pvidfilename = PL300_PCIE_EVK_VID_FW_FILENAME;
            paudfilename = PL300_AUD_FW_FILENAME;
            pDevice->m_initData.mpgCodecInitData.AccessMode = QPHCI_MODE_DIRECT;
            pDevice->m_initData.mpgCodecInitData.BusType = QPHCI_BUS_PCI;
            pDevice->m_initData.mpgCodecInitData.MemType = QPCODEC_MEM_TYPE_mDDR;
            pDevice->m_initData.mpgCodecInitData.MemSize = 512;
            pDevice->m_initData.mpgCodecInitData.ChipType = QPCODEC_CHIP_ARTESA;

            pDevice->m_initData.mpgCodecInitData.BusData = pDevice->m_device.p_pciedev;
            pDevice->m_initData.mpgCodecInitData.BusDataSize = sizeof (struct pci_dev);
            #if 0 
			//GPIODirections and GPIOValues seems like unuse
            /* PCIE Device GPIO Default Directions and Values */
            pDevice->m_initData.mpgCodecInitData.GPIODirections = 0xc1db;
            /* HW Default 0100000001011011 */
            /* SPEC       1100000111011011 */
            pDevice->m_initData.mpgCodecInitData.GPIOValues = 0xc1db;
            pDevice->m_initData.audCodecInitData.type = AUDCODEC_AIC3101;

            #else

            /*7601************************/
        	pDevice->m_initData.audCodecInitData.type               = AUDCODEC_DUMMY;
            pDevice->m_initData.mpgCodecInitData.GPIODirections     = 0x7002;
            /* HW Default ???????????????? */
            /* SPEC       ???????????????? */
            pDevice->m_initData.mpgCodecInitData.GPIOValues         = 0x1002;
            /*7601************************/
            #endif


            pDevice->m_initData.mpgCodecInitData.ErrorRecovery = QPCODEC_ERROR_RECOVERY_ALWAYS;

            pDevice->m_initData.vidDecoderInitData.type = VIDDECODER_DUMMY;
            pDevice->m_initData.vidDecoderInitData.channel = VIDEO_SOURCE_DIGITAL1;
            pDevice->m_initData.vidDecoderInitData.multiType = MULTIPLE_VIDDECODER_NONE;
            pDevice->m_initData.vidEncoderInitData.type = VIDENCODER_DUMMY;
            pDevice->m_initData.vidEncoderInitData.standard = VIDEO_STANDARD_NTSC;
            pDevice->m_initData.tunerInitData.type = TUNER_DUMMY;
            pDevice->m_initData.tvAudioInitData.type = TVAUDIO_DUMMY;
            // i2c type use gpio i2c
            pDevice->m_initData.i2cInitData.type = QPI2C_TYPE_SOC_INTERNAL_GPIO;
            pDevice->m_initData.i2cInitData.clk = 14;
            pDevice->m_initData.i2cInitData.data = 15;
            pDevice->m_initData.i2cInitDataEx.type = QPI2C_TYPE_DUMMY;
            pDevice->m_initData.i2cInitDataEx.clk = 14;
            pDevice->m_initData.i2cInitDataEx.data = 15;
            break;

        default:
            return (QP_NULL);
        }

#if defined(CONFIG_QL300_AVIU_SIL9025)
        pDevice->m_initData.vidDecoderInitData.type = VIDDECODER_SIL9025;
#elif defined(CONFIG_QL300_AVIU_ADV7614)
        pDevice->m_initData.vidDecoderInitData.type = VIDDECODER_ADV7614;
#elif defined(CONFIG_QL300_AVIU_ADV7604)
    #if 0 
            pDevice->m_initData.vidDecoderInitData.type = VIDDECODER_ADV7604;
    #else
            pDevice->m_initData.vidDecoderInitData.type = VIDDECODER_GV7601;
    #endif
#elif defined(CONFIG_QL300_AVIU_ADI7401)
        pDevice->m_initData.vidDecoderInitData.type = VIDDECODER_ADI7401;
#elif defined(CONFIG_QL300_AVIU_PE1005)
        pDevice->m_initData.vidDecoderInitData.type = VIDDECODER_PE1005;
#elif defined(CONFIG_QL300_AVIU_ADI7441)
        pDevice->m_initData.vidDecoderInitData.type = VIDDECODER_ADI7441;
#elif defined(CONFIG_QL300_AVIU_IT6603)
        pDevice->m_initData.vidDecoderInitData.type = VIDDECODER_IT6603;
#elif defined(CONFIG_QL300_AVIU_OV5642)
        pDevice->m_initData.vidDecoderInitData.type = VIDDECODER_OV5642;
        pDevice->m_initData.mpgCodecInitData.GPIODirections = pDevice->m_initData.mpgCodecInitData.GPIODirections | (1 << 0) | (1 << 1); //gpio0 and gpio1 output high
        pDevice->m_initData.mpgCodecInitData.GPIOValues = pDevice->m_initData.mpgCodecInitData.GPIOValues | (1 << 0) | (1 << 1);
#elif defined(CONFIG_QL300_AVIU_OV10633)
        pDevice->m_initData.vidDecoderInitData.type = VIDDECODER_OV10633;
#elif defined(CONFIG_QL300_AVIU_NATIVE_MI2010)
        pDevice->m_initData.vidDecoderInitData.type = VIDDECODER_NATIVE_MI2010;
#endif

#if defined(CONFIG_QL300_AVOU_ADI7393)
        pDevice->m_initData.vidEncoderInitData.type = VIDENCODER_ADI7393;
#elif defined(CONFIG_QL300_AVOU_SIL9034)
        pDevice->m_initData.vidEncoderInitData.type = VIDENCODER_SIL9034;
#elif defined(CONFIG_QL300_AVOU_ADI9889)
        pDevice->m_initData.vidEncoderInitData.type = VIDENCODER_ADI9889;
#endif

#if defined(CONFIG_HOST_PLATFORM_PL330SOC)
        pDevice->m_initData.vidDecoderInitData.type = pl330_frondend_type;
#endif

        pDevice->m_initData.mpgCodecInitData.RegisterBase = RegisterBase;
        pDevice->m_initData.mpgCodecInitData.MemoryBase = MemoryBase;
        pDevice->m_initData.mpgCodecInitData.PageSize = PageSize;
        pDevice->m_initData.mpgCodecInitData.InterruptNumber = InterruptNumber;

        pDevice->m_initData.mpgCodecInitData.VerFwAPI = 1;
        pDevice->m_initData.mpgCodecInitData.FwIntMode = 0;
#if defined(CONFIG_FW_STATIC_MEM_USAGE)
        QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
                     "%s()-> config to use Static Memory Allocation for FW\n",
                     __FUNCTION__
                     ));
        pDevice->m_initData.mpgCodecInitData.FwFixedMode = 0; // static mode
#else
        QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
                     "%s()-> config to use Dynamic Memory Allocation for FW\n",
                     __FUNCTION__
                     ));
        pDevice->m_initData.mpgCodecInitData.FwFixedMode = 1; // dynamic mode
#endif

        pDevice->m_initData.mpgCodecInitData.VIUMode = 2;
        pDevice->m_initData.mpgCodecInitData.VIUFormat = 1;
        pDevice->m_initData.mpgCodecInitData.VIUStartPixel = 0;
        pDevice->m_initData.mpgCodecInitData.VIUStartLine = 0;
        pDevice->m_initData.mpgCodecInitData.ClkEdge = 0;
        pDevice->m_initData.mpgCodecInitData.VIUSyncCode1 = 0xf1f1f1da;
        pDevice->m_initData.mpgCodecInitData.VIUSyncCode2 = 0xb6f1f1b6;

        pDevice->m_initData.mpgCodecInitData.AIControls_ai_msb = 1;
        pDevice->m_initData.mpgCodecInitData.AIControls_lrclk_i = 0;
        pDevice->m_initData.mpgCodecInitData.AIControls_bclk_i = 0;
        pDevice->m_initData.mpgCodecInitData.AIControls_ai_i2s = 1;
        pDevice->m_initData.mpgCodecInitData.AIControls_ai_rj = 0;
#ifdef  CONFIG_PL_AI_FIX_IN_MASTER
        pDevice->m_initData.mpgCodecInitData.AIControls_ai_m = 1;
#else
        pDevice->m_initData.mpgCodecInitData.AIControls_ai_m = 0;
#endif
        pDevice->m_initData.mpgCodecInitData.VOEnable = QP_TRUE;
        pDevice->m_initData.mpgCodecInitData.VOUMode = 0;
        pDevice->m_initData.mpgCodecInitData.VOUStartPixel = 1;
        pDevice->m_initData.mpgCodecInitData.VOUStartLine = 0;

        pDevice->m_initData.mpgCodecInitData.AOEnable = QP_TRUE;
        pDevice->m_initData.mpgCodecInitData.AOControls_ao_msb = 1;
        pDevice->m_initData.mpgCodecInitData.AOControls_lrclk_i = 1;
        pDevice->m_initData.mpgCodecInitData.AOControls_bclk_i = 0;
        pDevice->m_initData.mpgCodecInitData.AOControls_ao_i2s = 1;
        pDevice->m_initData.mpgCodecInitData.AOControls_ao_rj = 1;
#ifdef CONFIG_PL_AO_FIX_IN_SLAVE
        pDevice->m_initData.mpgCodecInitData.AOControls_ao_s = 1;
#else
        pDevice->m_initData.mpgCodecInitData.AOControls_ao_s = 0;
#endif

        if (pDevice->m_initData.mpgCodecInitData.bUseExtFW)
        {
#ifdef EMBED_FW
            extern u8 video_fw[];
            extern u8 audio_fw[];
            extern unsigned long video_fw_size;
            extern unsigned long audio_fw_size;
            pDevice->m_initData.mpgCodecInitData.pVideoFW=(void *)video_fw;
            pDevice->m_initData.mpgCodecInitData.VidFWSize=video_fw_size;
            pDevice->m_initData.mpgCodecInitData.pAudioFW=(void *)audio_fw;
            pDevice->m_initData.mpgCodecInitData.AudFWSize=audio_fw_size;
            printk("VideoFW %p size %d AudioFW %p size %d\n",video_fw,video_fw_size,audio_fw,audio_fw_size);

#else
            long lErr;
            lErr = qlf_load(pvidfilename,
                            &pDevice->m_initData.mpgCodecInitData.pVideoFW,
                            &pDevice->m_initData.mpgCodecInitData.VidFWSize
                            );
            if (!lErr)
            {
                lErr = qlf_load(paudfilename,
                                &pDevice->m_initData.mpgCodecInitData.pAudioFW,
                                &pDevice->m_initData.mpgCodecInitData.AudFWSize
                                );
            }

            if (lErr)
            {
                pDevice->m_initData.mpgCodecInitData.bUseExtFW = QP_FALSE;

                if (pDevice->m_initData.mpgCodecInitData.pVideoFW)
                {
#ifdef USE_KMALLOC_INSTEAD_OF_VMALLOC
                    kfree(pDevice->m_initData.mpgCodecInitData.pVideoFW);
#else
                    vfree(pDevice->m_initData.mpgCodecInitData.pVideoFW);
#endif
                    pDevice->m_initData.mpgCodecInitData.pVideoFW = QP_NULL;
                }
                if (pDevice->m_initData.mpgCodecInitData.pAudioFW)
                {
#ifdef USE_KMALLOC_INSTEAD_OF_VMALLOC
                    kfree(pDevice->m_initData.mpgCodecInitData.pAudioFW);
#else
                    vfree(pDevice->m_initData.mpgCodecInitData.pAudioFW);
#endif
                    pDevice->m_initData.mpgCodecInitData.pAudioFW = QP_NULL;
                }
            }
#endif
        }

        pDevice->m_PowerState = PLV4LDEV_PWR_STATE_D0;

        pDevice->m_pComponentMgr = QP_NULL;

        pDevice->m_pCodecLib = QP_NULL;
        pDevice->m_pMpegCodec = QP_NULL;
        pDevice->m_pVidDecoder = QP_NULL;
        pDevice->m_pVidEncoder = QP_NULL;
        pDevice->m_pTuner = QP_NULL;
        pDevice->m_pTVAudio = QP_NULL;
        pDevice->m_pAudCodec = QP_NULL;

#ifdef SUPPORT_PL3XX_OSD
        memset(&pDevice->m_OSDInfo, 0, sizeof (struct OSDInfo));
#endif
    }

    atomic_set(&pDevice->stream_started, 0);

    init_waitqueue_head(&pDevice->wq_qbuf);
    init_waitqueue_head(&pDevice->wq_dqbuf);

    pDevice->qbuf_queues = create_singlethread_workqueue("c353_qbuf_queue");
    pDevice->dqbuf_queues = create_singlethread_workqueue("c353_dqbuf_queue");

    INIT_WORK(&pDevice->wq_wait_qbuf, ccomponent_wait_qbuf);
    INIT_WORK(&pDevice->wq_wait_dqbuf, ccomponent_wait_dqbuf);

    return (pDevice);
}

void CDevice_Destructor(CDevice *This)
{
    int i;
#ifndef EMBED_FW
    if (This->m_initData.mpgCodecInitData.pVideoFW)
    {
#ifdef USE_KMALLOC_INSTEAD_OF_VMALLOC
        kfree(This->m_initData.mpgCodecInitData.pVideoFW);
#else
        vfree(This->m_initData.mpgCodecInitData.pVideoFW);
#endif
        This->m_initData.mpgCodecInitData.pVideoFW = QP_NULL;
    }

    if (This->m_initData.mpgCodecInitData.pAudioFW)
    {
#ifdef USE_KMALLOC_INSTEAD_OF_VMALLOC
        kfree(This->m_initData.mpgCodecInitData.pAudioFW);
#else
        vfree(This->m_initData.mpgCodecInitData.pAudioFW);
#endif
        This->m_initData.mpgCodecInitData.pAudioFW = QP_NULL;
    }
#endif

    if (This->m_pComponentMgr)
    {
        CObjectMgr_Destructor(This->m_pComponentMgr);
        kfree((void *) This->m_pComponentMgr);
        This->m_pComponentMgr = QP_NULL;
    }

    for (i = 0; i < MAX_COMPONENT_TYPE; i++)
    {
        if (This->m_pVideoDevices[i])
        {
            QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_WARN),
                         "%s()-> video_unregister_device type(%d) minor(%d)\n",
                         __FUNCTION__,
                         i,
                         This->m_pVideoDevices[i]->minor
                         ));
            video_unregister_device(This->m_pVideoDevices[i]);
            This->m_pVideoDevices[i] = QP_NULL;
        }
    }

    pl330_audio_unregister(This);

    flush_workqueue(This->qbuf_queues);
    flush_workqueue(This->dqbuf_queues);
    destroy_workqueue(This->qbuf_queues);
    destroy_workqueue(This->dqbuf_queues);

    CObject_Destructor(&This->m_Object);
}

EQP_Errors CDevice_Create(CDevice *This)
{
    EQP_Errors qpStatus;
    int i;
    int ret;

    QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_TRACE),
                 "%s\n",
                 __FUNCTION__
                 ));



    // create component manager
    //
    This->m_pComponentMgr = (CObjectMgr *) kmalloc(sizeof (CObjectMgr),
                                                   GFP_KERNEL
                                                   );
    if (!CObjectMgr_Constructor(This->m_pComponentMgr,
                                &This->m_Object,
                                OBJECT_CRITICAL_HEAVY,
                                QP_NULL
                                ))
    {
        QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
                     "%s() CObjectMgr_Constructor Failed!!!!\n",
                     __FUNCTION__
                     ));
        CDevice_Close(This);
        return (QPERR_NOMEM);
    }


    // initialize codec library
    //
    qpStatus = QPCodecInitLibrary(This->m_device.read,
                                  QP_NULL,
                                  &This->m_initData,
                                  &This->m_pCodecLib,
                                  This
                                  );

    // get pointers to library interfaces
    //
    qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPCodecLib_GetMpegCodec(This->m_pCodecLib, &This->m_pMpegCodec);
    qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPCodecLib_GetVideoEncoder(This->m_pCodecLib, &This->m_pVidEncoder);
    qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPCodecLib_GetTuner(This->m_pCodecLib, &This->m_pTuner);
    qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPCodecLib_GetTVAudio(This->m_pCodecLib, &This->m_pTVAudio);
    //qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPCodecLib_GetVideoDecoder(This->m_pCodecLib, &This->m_pVidDecoder);
    //qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPCodecLib_GetAudioCodec(This->m_pCodecLib, &This->m_pAudCodec);

    //	This->m_pCodecLib

    if (!QP_SUCCESS(qpStatus))
    {
        QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
                     "%s QPCodecInitLibrary() Failed() Status(%d)\n",
                     __FUNCTION__,
                     qpStatus
                     ));
        CDevice_Close(This);
        return (qpStatus);
    }

    pl330_trace(PL330_TRACE_CALL, "%s begin qpgenericdeviec initdevice\n", __func__);
    qpStatus = QPGenericDevice_InitDevice(This->m_pCodecLib,
                                          CDevice_Callback,
                                          This,
                                          QP_NULL
                                          );

    pl330_trace(PL330_TRACE_CALL, "%s end of qpgenericdeviec initdevice\n", __func__);
#ifndef EMBED_FW
    if (This->m_initData.mpgCodecInitData.pVideoFW)
    {
#ifdef USE_KMALLOC_INSTEAD_OF_VMALLOC
        kfree(This->m_initData.mpgCodecInitData.pVideoFW);
#else
        vfree(This->m_initData.mpgCodecInitData.pVideoFW);
#endif
        This->m_initData.mpgCodecInitData.pVideoFW = QP_NULL;
    }

    if (This->m_initData.mpgCodecInitData.pAudioFW)
    {
#ifdef USE_KMALLOC_INSTEAD_OF_VMALLOC
        kfree(This->m_initData.mpgCodecInitData.pAudioFW);
#else
        vfree(This->m_initData.mpgCodecInitData.pAudioFW);
#endif
        This->m_initData.mpgCodecInitData.pAudioFW = QP_NULL;
    }
#endif


    // allocate and register V4L device
    //
    for (i = 0; i < MAX_COMPONENT_TYPE; i++)
    {

#if LINUX_VERSION_CODE >= KERNEL_VERSION(3,11,0)
        if (v4l2_device_register(This->m_pGenericDevice,
                                 &This->vdev[i]) < 0)
        {
            pl330_trace(PL330_TRACE_CALL, "%s register v4l2 device fail\n",
                        __func__);
            return QPERR_FAIL;
        }
#endif
        This->m_pVideoDevices[i] = video_device_alloc();
        if (This->m_pVideoDevices[i])
        {
            memcpy(This->m_pVideoDevices[i],
                   &plv4l_videodevice,
                   sizeof (struct video_device)
                   );

            video_set_drvdata(This->m_pVideoDevices[i],
                              NULL
                              );

#if LINUX_VERSION_CODE >= KERNEL_VERSION(3,11,0)
            This->m_pVideoDevices[i]->v4l2_dev = &This->vdev[i];
            This->m_pVideoDevices[i]->dev_parent = This->m_pGenericDevice;
#elif LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,27)
            This->m_pVideoDevices[i]->parent = This->m_pGenericDevice;
#else //LINUX_VERSION_CODE < KERNEL_VERSION(2,6,27)
            This->m_pVideoDevices[i]->dev = This->m_pGenericDevice;
#endif //LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,27)
            This->m_pVideoDevices[i]->release = video_device_release;


            ret = video_register_device(This->m_pVideoDevices[i],
                                        VFL_TYPE_GRABBER,
                                        -1
                                        );
            if (ret < 0)
            {
                QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
                             "%s()-> Unable to register v4l type(%d)\n",
                             __FUNCTION__,
                             i
                             ));
                video_device_release(This->m_pVideoDevices[i]);
                This->m_pVideoDevices[i] = QP_NULL;
            }
            else
            {
                QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_WARN),
                             "%s()-> Registered v4l type(%d) minor(%d)\n",
                             __FUNCTION__,
                             i,
                             This->m_pVideoDevices[i]->minor
                             ));
            }
        }
    }

    /* register ALSA device */
    This->m_pAudioDev = pl330_audio_register(This);

    pl330_trace(PL330_TRACE_CALL, "end of %s\n", __func__);

    if (!QP_SUCCESS(qpStatus))
    {
        CDevice_Close(This);
        return (qpStatus);
    }

    CObject_Init(&This->m_Object);
    return (qpStatus);
}

EQP_Errors CDevice_Close(CDevice *This)
{
    int i;

    QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_TRACE),
                 "%s\n",
                 __FUNCTION__
                 ));

    // close all the remaining components
    //
    while (0 != CObjectMgr_GetNumberOfObjects(This->m_pComponentMgr))
    {
        CComponent *pComponent = CObjectMgr_GetObjectByIndex(This->m_pComponentMgr,
                                                             0
                                                             );
        if (pComponent)
        {
            CComponent_Close(pComponent);
            CComponent_Destructor(pComponent);
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,24)) && !defined(CONFIG_HOST_PLATFORM_PL330SOC)
            kfree((void *) pComponent);
#else
            vfree((void *) pComponent);
#endif
        }
    }

    // unregister video device
    //
    for (i = 0; i < MAX_COMPONENT_TYPE; i++)
    {
        if (This->m_pVideoDevices[i])
        {
            QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_WARN),
                         "%s()-> video_unregister_device type(%d) minor(%d)\n",
                         __FUNCTION__,
                         i,
                         This->m_pVideoDevices[i]->minor
                         ));
            video_unregister_device(This->m_pVideoDevices[i]);
            This->m_pVideoDevices[i] = QP_NULL;
        }
    }

    // release all interfaces we have acquired
    //
    if (This->m_pAudCodec)
    {
        QPGenericDevice_Release(This->m_pAudCodec);
        This->m_pAudCodec = QP_NULL;
    }

    if (This->m_pTVAudio)
    {
        QPGenericDevice_Release(This->m_pTVAudio);
        This->m_pTVAudio = QP_NULL;
    }

    if (This->m_pTuner)
    {
        QPGenericDevice_Release(This->m_pTuner);
        This->m_pTuner = QP_NULL;
    }

    if (This->m_pVidEncoder)
    {
        QPGenericDevice_Release(This->m_pVidEncoder);
        This->m_pVidEncoder = QP_NULL;
    }

    if (This->m_pVidDecoder)
    {
        QPCodecLib_GetVideoDecoder(This->m_pCodecLib, &This->m_pVidDecoder);

        QPGenericDevice_Release(This->m_pVidDecoder);
        This->m_pVidDecoder = QP_NULL;
    }

    if (This->m_pMpegCodec)
    {
        QPGenericDevice_Release(This->m_pMpegCodec);
        This->m_pMpegCodec = QP_NULL;
    }

    if (This->m_pCodecLib)
    {
        QPGenericDevice_Release(This->m_pCodecLib);
        QPCodecDoneLibrary(This->m_pCodecLib);
        This->m_pCodecLib = QP_NULL;
    }

    CObject_Done(&This->m_Object);

    return (QPERR_SUCCESS);
}

EQP_Errors CDevice_PowerUp(CDevice *This)
{
    EQP_Errors qpStatus = QPERR_SUCCESS;

    if (QP_NULL != This->m_pCodecLib)
    {
        qpStatus = This->m_pCodecLib->Enable(This->m_pCodecLib);
    }
    This->m_PowerState = PLV4LDEV_PWR_STATE_D0;
    return (qpStatus);
}

EQP_Errors CDevice_PowerDown(CDevice *This)
{
    EQP_Errors qpStatus = QPERR_SUCCESS;

    if (QP_NULL != This->m_pCodecLib)
    {
        qpStatus = This->m_pCodecLib->Disable(This->m_pCodecLib);
    }
    This->m_PowerState = PLV4LDEV_PWR_STATE_D3;
    return (qpStatus);
}



/////////////////////////////////////////////////////////////////////////////
//
//

// low level hardware access functions
//

long CDevice_rdmem(CDevice *This,
                   unsigned long ulAddr,
                   unsigned long ulSize,
                   unsigned char *pucaBuf,
                   int isusrbuf
                   )
{
    ICodecLib *pCodecLib = This->m_pCodecLib;
    EQP_Errors qpStatus = QPERR_SUCCESS;
    QPCODEC_DIAG_MEMORY_STRUCT mem;
    TQP_UINT32 dwGot = 0;
    int nDwordCount = ulSize / 4;
    int nXfer;
    unsigned long ulOffset = 0;

    while (nDwordCount && QP_SUCCESS(qpStatus))
    {
        nXfer = aver_min(nDwordCount, QP_MAX_MEMORY_DWORDS);

        mem.address = ulAddr + ulOffset;
        mem.num_dwords = nXfer;

        qpStatus = pCodecLib->Get(pCodecLib,
                                  (TQP_REFGUID) & PROPSETID_QPCODEC_DIAG,
                                  TQP_NULL_HANDLE,
                                  QPCODEC_DIAG_MEM_READ,
                                  (TQP_PVOID) & mem,
                                  (TQP_PVOID) & mem,
                                  &dwGot
                                  );
        if (QP_SUCCESS(qpStatus))
        {
            if (isusrbuf)
            {
                copy_to_user(pucaBuf + ulOffset,
                             (unsigned char *) &mem.value[0],
                             nXfer * 4
                             );
            }
            else
            {
                memcpy(pucaBuf + ulOffset,
                       (unsigned char *) &mem.value[0],
                       nXfer * 4
                       );
            }
        }

        nDwordCount -= nXfer;
        ulOffset += (nXfer * 4);
    }

    return (QpToLinuxKerStatus(qpStatus));
}

long CDevice_wrmem(CDevice *This,
                   void *addr,
                   unsigned long ulSize,
                   unsigned char *pucaBuf,
                   int isusrbuf
                   )
{
    ICodecLib *pCodecLib = This->m_pCodecLib;
    EQP_Errors qpStatus = QPERR_SUCCESS;
    QPCODEC_DIAG_MEMORY_STRUCT mem;
    int nDwordCount = ulSize / 4;
    int nXfer;
    unsigned long ulOffset = 0;
    unsigned long ulAddr = (unsigned long) addr;

    while (nDwordCount && QP_SUCCESS(qpStatus))
    {
        nXfer = aver_min(nDwordCount, QP_MAX_MEMORY_DWORDS);

        mem.address = ulAddr + ulOffset;
        mem.num_dwords = nXfer;

        if (isusrbuf)
        {
            copy_from_user((unsigned char *) &mem.value[0],
                           pucaBuf + ulOffset,
                           nXfer * 4
                           );
        }
        else
        {
            memcpy((unsigned char *) &mem.value[0],
                   pucaBuf + ulOffset,
                   nXfer * 4
                   );
        }

        qpStatus = pCodecLib->Set(pCodecLib,
                                  (TQP_REFGUID) & PROPSETID_QPCODEC_DIAG,
                                  TQP_NULL_HANDLE,
                                  QPCODEC_DIAG_MEM_WRITE,
                                  (TQP_PVOID) & mem,
                                  QP_NULL,
                                  sizeof (QPCODEC_DIAG_MEMORY_STRUCT)
                                  );

        nDwordCount -= nXfer;
        ulOffset += (nXfer * 4);
    }

    return (QpToLinuxKerStatus(qpStatus));
}

long CDevice_dmard(CDevice *This,
                   void *addr,
                   unsigned long ulSize,
                   unsigned char *pucaBuf,
                   int isusrbuf
                   )
{
    ICodecLib *pCodecLib = This->m_pCodecLib;
    EQP_Errors qpStatus;
    QPCODEC_DIAG_DMA_PTR_STRUCT dma;
    TQP_UINT32 dwGot = 0;
    unsigned long ulAddr = (unsigned long) (uintptr_t) addr;

    if (isusrbuf)
    {
#ifdef USE_KMALLOC_INSTEAD_OF_VMALLOC
        dma.pData = kmalloc(ulSize, GFP_KERNEL);
#else
        dma.pData = vmalloc(ulSize);
#endif
    }
    else
    {
        dma.pData = pucaBuf;
    }

    if (!dma.pData)
    {
        QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
                     "%s() vmalloc %d bytes failed\n",
                     __FUNCTION__,
                     ulSize
                     ));
        return (-ENOMEM);
    }

    dma.deviceAddress = ulAddr;
    dma.size = ulSize;

    qpStatus = pCodecLib->Get(pCodecLib,
                              (TQP_REFGUID) & PROPSETID_QPCODEC_DIAG,
                              TQP_NULL_HANDLE,
                              QPCODEC_DIAG_DMA_READ_PTR,
                              (TQP_PVOID) & dma,
                              (TQP_PVOID) & dma,
                              &dwGot
                              );
    if (isusrbuf)
    {
        if (QP_SUCCESS(qpStatus))
        {
            copy_to_user(pucaBuf,
                         dma.pData,
                         ulSize
                         );
        }
#ifdef USE_KMALLOC_INSTEAD_OF_VMALLOC
        kfree(dma.pData);
#else
        vfree(dma.pData);
#endif
    }

    return (QpToLinuxKerStatus(qpStatus));
}

long CDevice_dmawr(CDevice *This,
                   unsigned long ulAddr,
                   unsigned long ulSize,
                   unsigned char *pucaBuf,
                   int isusrbuf
                   )
{
    ICodecLib *pCodecLib = This->m_pCodecLib;
    EQP_Errors qpStatus;
    QPCODEC_DIAG_DMA_PTR_STRUCT dma;

    if (isusrbuf)
    {
#ifdef USE_KMALLOC_INSTEAD_OF_VMALLOC
        dma.pData = kmalloc(ulSize, GFP_KERNEL);
#else
        dma.pData = vmalloc(ulSize);
#endif
        if (dma.pData)
        {
            copy_from_user(dma.pData,
                           pucaBuf,
                           ulSize
                           );
        }
    }
    else
    {
        dma.pData = pucaBuf;
    }

    if (!dma.pData)
    {
        QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
                     "%s() vmalloc %d bytes failed\n",
                     __FUNCTION__,
                     ulSize
                     ));
        return (-ENOMEM);
    }

    dma.deviceAddress = ulAddr;
    dma.size = ulSize;

    qpStatus = pCodecLib->Set(pCodecLib,
                              (TQP_REFGUID) & PROPSETID_QPCODEC_DIAG,
                              TQP_NULL_HANDLE,
                              QPCODEC_DIAG_DMA_WRITE_PTR,
                              (TQP_PVOID) & dma,
                              QP_NULL,
                              sizeof (QPCODEC_DIAG_DMA_PTR_STRUCT)
                              );

    if (isusrbuf)
    {
#ifdef USE_KMALLOC_INSTEAD_OF_VMALLOC
        kfree(dma.pData);
#else
        vfree(dma.pData);
#endif
    }

    return (QpToLinuxKerStatus(qpStatus));
}

long CDevice_rdreg(CDevice *This,
                   unsigned long ulAddr,
                   unsigned char *pucaBuf,
                   int isusrbuf
                   )
{
    ICodecLib *pCodecLib = This->m_pCodecLib;
    EQP_Errors qpStatus;
    QPCODEC_DIAG_REGISTER_STRUCT reg;
    TQP_UINT32 dwGot = 0;

    reg.address = ulAddr;

    qpStatus = pCodecLib->Get(pCodecLib,
                              (TQP_REFGUID) & PROPSETID_QPCODEC_DIAG,
                              TQP_NULL_HANDLE,
                              QPCODEC_DIAG_REG_READ,
                              (TQP_PVOID) & reg,
                              (TQP_PVOID) & reg,
                              &dwGot
                              );
    if (QP_SUCCESS(qpStatus))
    {
        if (isusrbuf)
        {
            copy_to_user(pucaBuf,
                         (unsigned char *) &reg.value,
                         4
                         );
        }
        else
        {
            memcpy(pucaBuf,
                   (unsigned char *) &reg.value,
                   4
                   );
        }
    }

    return (QpToLinuxKerStatus(qpStatus));
}

long CDevice_wrreg(CDevice *This,
                   unsigned long ulAddr,
                   unsigned char *pucaBuf,
                   int isusrbuf
                   )
{
    ICodecLib *pCodecLib = This->m_pCodecLib;
    EQP_Errors qpStatus;
    QPCODEC_DIAG_REGISTER_STRUCT reg;

    if (isusrbuf)
    {
        copy_from_user((unsigned char *) &reg.value,
                       pucaBuf,
                       4
                       );
    }
    else
    {
        memcpy((unsigned char *) &reg.value,
               pucaBuf,
               4
               );
    }

    reg.address = ulAddr;

    qpStatus = pCodecLib->Set(pCodecLib,
                              (TQP_REFGUID) & PROPSETID_QPCODEC_DIAG,
                              TQP_NULL_HANDLE,
                              QPCODEC_DIAG_REG_WRITE,
                              (TQP_PVOID) & reg,
                              QP_NULL,
                              sizeof (QPCODEC_DIAG_REGISTER_STRUCT)
                              );

    return (QpToLinuxKerStatus(qpStatus));
}

long CDevice_rdhci(CDevice *This,
                   unsigned long ulAddr,
                   unsigned char *pucaBuf,
                   int isusrbuf
                   )
{
    ICodecLib *pCodecLib = This->m_pCodecLib;
    EQP_Errors qpStatus;
    QPCODEC_DIAG_REGISTER_STRUCT reg;
    TQP_UINT32 dwGot = 0;

    reg.address = ulAddr;
    reg.value = 0;

    qpStatus = pCodecLib->Get(pCodecLib,
                              (TQP_REFGUID) & PROPSETID_QPCODEC_DIAG,
                              TQP_NULL_HANDLE,
                              QPCODEC_DIAG_HCI_REG_READ,
                              (TQP_PVOID) & reg,
                              (TQP_PVOID) & reg,
                              &dwGot
                              );
    if (QP_SUCCESS(qpStatus))
    {
        if (isusrbuf)
        {
            copy_to_user(pucaBuf,
                         (unsigned char *) &reg.value,
                         4
                         );
        }
        else
        {
            memcpy(pucaBuf,
                   (unsigned char *) &reg.value,
                   4
                   );
        }
    }

    return (QpToLinuxKerStatus(qpStatus));
}

long CDevice_wrhci(CDevice *This,
                   unsigned long ulAddr,
                   unsigned char *pucaBuf,
                   int isusrbuf
                   )
{
    ICodecLib *pCodecLib = This->m_pCodecLib;
    EQP_Errors qpStatus;
    QPCODEC_DIAG_REGISTER_STRUCT reg;

    if (isusrbuf)
    {
        copy_from_user((unsigned char *) &reg.value,
                       pucaBuf,
                       4
                       );
    }
    else
    {
        memcpy((unsigned char *) &reg.value,
               pucaBuf,
               4
               );
    }

    reg.address = ulAddr;

    qpStatus = pCodecLib->Set(pCodecLib,
                              (TQP_REFGUID) & PROPSETID_QPCODEC_DIAG,
                              TQP_NULL_HANDLE,
                              QPCODEC_DIAG_HCI_REG_WRITE,
                              (TQP_PVOID) & reg,
                              QP_NULL,
                              sizeof (QPCODEC_DIAG_REGISTER_STRUCT)
                              );

    return (QpToLinuxKerStatus(qpStatus));
}

long CDevice_rdi2c(CDevice *This,
                   unsigned char ucDevAddr,
                   unsigned long ulRegSize,
                   unsigned char *pucRegAddr,
                   unsigned long ulSize,
                   unsigned char *pucaBuf,
                   int isusrbuf
                   )
{
    ICodecLib *pCodecLib = This->m_pCodecLib;
    EQP_Errors qpStatus = QPERR_SUCCESS;
    QPCODEC_DIAG_I2C_WRITE_THEN_READ_STRUCT i2c;
    TQP_UINT32 dwGot = 0;

    i2c.chip_address = ucDevAddr;
    i2c.num_write_bytes = ulRegSize;
    if (ulRegSize)
    {
        memcpy((unsigned char *) &i2c.write_bytes[0],
               pucRegAddr,
               ulRegSize
               );
    }
    i2c.num_read_bytes = ulSize;

    qpStatus = pCodecLib->Get(pCodecLib,
                              (TQP_REFGUID) & PROPSETID_QPCODEC_DIAG,
                              TQP_NULL_HANDLE,
                              QPCODEC_DIAG_I2C_WRITE_THEN_READ,
                              (TQP_PVOID) & i2c,
                              (TQP_PVOID) & i2c,
                              &dwGot
                              );
    if (QP_SUCCESS(qpStatus))
    {
        if (isusrbuf)
        {
            copy_to_user(pucaBuf,
                         (unsigned char *) &i2c.read_bytes[0],
                         ulSize
                         );
        }
        else
        {
            memcpy(pucaBuf,
                   (unsigned char *) &i2c.read_bytes[0],
                   ulSize
                   );
        }
    }

    return (QpToLinuxKerStatus(qpStatus));
}

long CDevice_wri2c(CDevice *This,
                   unsigned char ucDevAddr,
                   unsigned long ulSize,
                   unsigned char *pucaBuf,
                   int isusrbuf
                   )
{
    ICodecLib *pCodecLib = This->m_pCodecLib;
    EQP_Errors qpStatus = QPERR_SUCCESS;
    QPCODEC_DIAG_I2C_STRUCT i2c;


    if (!This)
    {
        printk("%s CDevice is NULL\n", __func__);
        return 0;
    }

    if (!pCodecLib)
    {
        printk("%s pCodecLib is NULL\n", __func__);
        return 0;
    }


    if (isusrbuf)
    {
        copy_from_user((unsigned char *) &i2c.bytes[0],
                       pucaBuf,
                       ulSize
                       );
    }
    else
    {
        memcpy((unsigned char *) &i2c.bytes[0],
               pucaBuf,
               ulSize
               );
    }

    qpStatus = pCodecLib->Set(pCodecLib,
                              (TQP_REFGUID) & PROPSETID_QPCODEC_DIAG,
                              TQP_NULL_HANDLE,
                              QPCODEC_DIAG_I2C_WRITE,
                              (TQP_PVOID) & i2c,
                              QP_NULL,
                              sizeof (QPCODEC_DIAG_I2C_STRUCT)
                              );

    return (QpToLinuxKerStatus(qpStatus));
}

