///////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2011 Sigma Designs, Inc.  All Rights Reserved.   
//
// Cdevice.h
//
// Description: 
//
//  Device class
// 
// History:
//	Dec 17, 2009	(dt) created
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////

#ifndef __CDEVICE_H__
#define __CDEVICE_H__


#include <linux/types.h>
#include <linux/file.h>
#include <linux/fs.h>
#include <linux/timer.h>
#include <linux/list.h>
#include <linux/pci.h>
#include <linux/wait.h>
#include <linux/sched.h>


#include <linux/version.h>

#if LINUX_VERSION_CODE >= KERNEL_VERSION(3,11,0)
#include <media/v4l2-device.h>
#endif
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,18)
#include <media/v4l2-dev.h>
#endif

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,27)
#include <media/v4l2-ioctl.h>
#endif

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,26)
#include <linux/semaphore.h>
#else
#include <asm/semaphore.h>
#endif


#ifdef __LINUX24__
#include "videodev2.h"
#else
#include <linux/videodev2.h>
#endif

#include "qptypes.h"
#include "qpcodeclib.h"
#include "qpmpgcodec.h"
#include "cobject.h"
#include "cobjectmgr.h"
#include "ccomponent.h"

#ifdef SUPPORT_PL3XX_OSD
#include "qlosd.h"

struct OSDInfo {
	OSD_DISPLAY_INFO mDisplay;
	unsigned long unFrameBufferAddr;
	#ifdef SUPPORT_VI_OSD
	QPVIOSD_CONTROL mViosdDisplay;
	unsigned int mViosdON;
	#endif
};
#endif

/////////////////////////////////////////////////////////////////////////////
//
//


typedef enum _PLV4LDEV_PowerStates
{   
    PLV4LDEV_PWR_STATE_D0	= 0,	// on
    PLV4LDEV_PWR_STATE_D1	,		// stand by
    PLV4LDEV_PWR_STATE_D2	,		// hibernate
    PLV4LDEV_PWR_STATE_D3			// off
} PLV4LDEV_PowerStates, *PPLV4LDEV_PowerStates;


typedef enum _PLV4LDEV_TYPE
{
	PLV4LDEV_TYPE_NONE		= 0,
	PLV4LDEV_TYPE_QL201_FX2,
	PLV4LDEV_TYPE_QL300_FX2,
	PLV4LDEV_TYPE_QL201_USB_EVK,
	PLV4LDEV_TYPE_QL300_USB_EVB,
	PLV4LDEV_TYPE_QL300_USB_EVK,
	PLV4LDEV_TYPE_QL300_USB_CAM,
	PLV4LDEV_TYPE_QL300_USB_CUS1,
	PLV4LDEV_TYPE_QL300_USB_CUS2,
	PLV4LDEV_TYPE_QL300_USB_CUS3,
	PLV4LDEV_TYPE_QL300_USB_CUS4,
	PLV4LDEV_TYPE_QL300_USB_FC3,
	PLV4LDEV_TYPE_QL300_STR_EVK,
	PLV4LDEV_TYPE_QL300_PCIE_EVK,
	PLV4LDEV_TYPE_QL300_MASTER,
	PLV4LDEV_TYPE_END

} PLV4LDEV_TYPE, *PPLV4LDEV_TYPE;

#define MAX_PLV4LDEV_TYPE	PLV4LDEV_TYPE_END


typedef union _PLV4LDEV_PARENT_DEV
{
#ifndef __LINUX24__
	struct usb_interface	*p_usbintf;
#else //__LINUX24__
	struct usb_device		*p_usbdev;
#endif //__LINUX24__
	struct pci_dev			*p_pciedev;
	struct device			*p_dev;
	void					*read;

} PLV4LDEV_PARENT_DEV, *PPLV4LDEV_PARENT_DEV;



/////////////////////////////////////////////////////////////////////////////
//
//


#ifndef __CDEVICE_FWD_DEFINED__
#define __CDEVICE_FWD_DEFINED__
typedef struct CDevice CDevice;
#endif //!__CDEVICE_FWD_DEFINED__


struct CDevice 
{
	// public CObject
	//
	CObject					m_Object;

	//PQLPCIE_DEVICE			*m_pPQLPCIE_DEVICE;
	void					*mmio_base;

	CComponent				*m_pComponent;
	CComponent				*pcm_component;
	

	// CDevice
	//
	TQP_UINT32				m_id;
	PLV4LDEV_TYPE			m_type;
	PLV4LDEV_PARENT_DEV		m_device;
	struct device			*m_pGenericDevice;

	// V4L video devices
	//
    struct video_device		*m_pVideoDevices[MAX_COMPONENT_TYPE];

#if LINUX_VERSION_CODE >= KERNEL_VERSION(3,11,0)
	struct v4l2_device		vdev[MAX_COMPONENT_TYPE];
#endif

	/* ALSA audio device */
	struct pl330_audio_dev	*m_pAudioDev;

	// init data
	//
	QPCODECLIB_INITDATA		m_initData;

	// saved resource list
	//
	QPCODEC_USB_BUS_DATA	m_usbBusData;

	// device power state
	//
	PLV4LDEV_PowerStates	m_PowerState;

	// component manager
	//
	CObjectMgr				*m_pComponentMgr;

	// codec interface objects goes here
	//
	ICodecLib				*m_pCodecLib;
	IMpegCodec				*m_pMpegCodec;
	IVideoDecoder			*m_pVidDecoder;
	IVideoEncoder			*m_pVidEncoder;
	ITuner					*m_pTuner;
	ITVAudio				*m_pTVAudio;
	IAudioCodec				*m_pAudCodec;

	atomic_t				stream_started;

	/* workqueue to check streaming */
	struct work_struct		wq_wait_qbuf;
	struct workqueue_struct	*qbuf_queues;
        bool	 qbuf_event;

	struct work_struct		wq_wait_dqbuf;
	struct workqueue_struct	*dqbuf_queues;
        bool	 dqbuf_event;

	wait_queue_head_t		wq_qbuf;
	wait_queue_head_t		wq_dqbuf;

	TQP_UINT32				v4l2_port_type;
	TQP_UINT32				audio_port_type;

#ifdef SUPPORT_PL3XX_OSD
	struct OSDInfo	m_OSDInfo;
#endif

}; 



CDevice * CDevice_Constructor(CDevice *pDevice, 
							  TQP_UINT32 id,
							  PLV4LDEV_TYPE type,
							  PPLV4LDEV_PARENT_DEV pDev,
							  struct device *pGenericDevice,
							  TQP_PVOID RegisterBase,
							  TQP_PVOID MemoryBase,
							  TQP_UINT32 PageSize,
							  TQP_UINT32 InterruptNumber
							  );
void CDevice_Destructor(CDevice *This);
EQP_Errors CDevice_Create(CDevice *This);
EQP_Errors CDevice_Close(CDevice *This);
EQP_Errors CDevice_PowerUp(CDevice *This);
EQP_Errors CDevice_PowerDown(CDevice *This);

CComponent *snd_pl330_component_open(CDevice *This);
void snd_pl330_component_close(CDevice *This, CComponent *component);


// access functions
//
static inline ICodecLib * CDevice_getCodecLib(CDevice *This) {return (This->m_pCodecLib);}
static inline IMpegCodec * CDevice_getMpegCodec(CDevice *This) {return (This->m_pMpegCodec);}
static inline IVideoDecoder * CDevice_getVideoDecoder(CDevice *This) {return (This->m_pVidDecoder);}
static inline IVideoEncoder * CDevice_getVideoEncoder(CDevice *This) {return (This->m_pVidEncoder);}
static inline ITuner * CDevice_getTuner(CDevice *This) {return (This->m_pTuner);}
static inline ITVAudio * CDevice_getTVAudio(CDevice *This) {return (This->m_pTVAudio);}
static inline IAudioCodec * CDevice_getAudioCodec(CDevice *This) {return (This->m_pAudCodec);}


#define PLV4LDEV_MAX_DEVICES	24
extern CDevice *g_pPlV4lDevices[PLV4LDEV_MAX_DEVICES];


/////////////////////////////////////////////////////////////////////////////
//
//

// low level hardware access functions
//
long CDevice_rdmem(CDevice *This, 
				   unsigned long ulAddr, 
				   unsigned long ulSize, 
				   unsigned char *pucaBuf, 
				   int isusrbuf
				   );
long CDevice_wrmem(CDevice *This, 
				   void *Addr, 
				   unsigned long ulSize, 
				   unsigned char *pucaBuf, 
				   int isusrbuf
				   );
long CDevice_dmard(CDevice *This, 
				   void *Addr, 
				   unsigned long ulSize, 
				   unsigned char *pucaBuf, 
				   int isusrbuf
				   );
long CDevice_dmawr(CDevice *This, 
				   unsigned long ulAddr, 
				   unsigned long ulSize, 
				   unsigned char *pucaBuf, 
				   int isusrbuf
				   );
long CDevice_rdreg(CDevice *This, 
				   unsigned long ulAddr, 
				   unsigned char *pucaBuf, 
				   int isusrbuf
				   );
long CDevice_wrreg(CDevice *This, 
				   unsigned long ulAddr, 
				   unsigned char *pucaBuf, 
				   int isusrbuf
				   );
long CDevice_rdhci(CDevice *This, 
				   unsigned long ulAddr, 
				   unsigned char *pucaBuf, 
				   int isusrbuf
				   );
long CDevice_wrhci(CDevice *This, 
				   unsigned long ulAddr, 
				   unsigned char *pucaBuf, 
				   int isusrbuf
				   );
long CDevice_rdi2c(CDevice *This,
				   unsigned char ucDevAddr, 
				   unsigned long ulRegSize, 
				   unsigned char *pucRegAddr, 
				   unsigned long ulSize, 
				   unsigned char *pucaBuf, 
				   int isusrbuf
				   );
long CDevice_wri2c(CDevice *This,
				   unsigned char ucDevAddr, 
				   unsigned long ulSize, 
				   unsigned char *pucaBuf, 
				   int isusrbuf
				   );

#endif //__CDEVICE_H__



