#include "pl330.h"
#include "frontend.h"
#include "boardinfo-probe.h"


void frontend_init(IFrontend *frontend,frontend_setup_t *setup_info)
{
    int i;
    if(frontend && setup_info)
    {
        for(i=0;i<setup_info->input_count;i++)
        {
            frontend->input_info[i].name=setup_info->input_name[i];
            frontend->input_info[i].status=FRONTEND_NO_SIGNAL;
        }
        frontend->input_count=setup_info->input_count;
        
    }  
}

void frontend_register_event_callback(IFrontend *frontend,frontend_event_cb_t cb_func,void *cb_cxt)
{
    if(frontend)
    {
        frontend->event_cb_func=cb_func;
        frontend->cb_cxt=cb_cxt;
    }
}

void frontend_update_event(IFrontend *frontend,IFrontend_event_e event)
{
    if(frontend)
    {
        switch(event)
        {
        case FRONTEND_EVENT_NO_SIGNAL:
        case FRONTEND_EVENT_NO_SYNC:
            
            break;
        case FRONTEND_EVENT_LOCK:
            if(frontend->ops)
            {
                if(frontend->ops->get_frameinfo)
                    frontend->ops->get_frameinfo(frontend,&frontend->input_frameinfo);
                if(frontend->ops->get_audioinfo)
                    frontend->ops->get_audioinfo(frontend,&frontend->audioinfo);
            }
            
            
            break;
        default:
            break;
        }
        if(frontend->streaming)
        {
            if(frontend->event_cb_func)
                frontend->event_cb_func(frontend->cb_cxt,event);
        }
    }
}

void frontend_start_streaming(IFrontend *frontend,TQP_BOOL start)
{
    if(frontend)
    {
        frontend->streaming=start;
        if(start)
        {
            frontend_input_status_e input_status=frontend_g_input_status(frontend);
            if(input_status==FRONTEND_LOCK)
                frontend_update_event(frontend,FRONTEND_EVENT_READY_STREAMING);
        }
    }    
}


int  frontend_s_ctrl(IFrontend *frontend, struct v4l2_control *ctrl)
{
    if(frontend)
    {
        if (frontend->ops->s_ctrl)
        {
            return frontend->ops->s_ctrl(frontend, ctrl);
        }
    }
    return -1;
}

int  frontend_g_ctrl(IFrontend *frontend, struct v4l2_control *ctrl)
{
    if(frontend)
    { 
        if (frontend->ops->g_ctrl)
        {
            return frontend->ops->g_ctrl(frontend, ctrl);
        }
    }
    return -1;
    
}

int  frontend_s_input(IFrontend *frontend, int index)
{
    if(frontend)
    { 
        if(index < frontend->input_count)
        {    
            int ret=0;
            if(frontend->cur_input!=index)
            {
                if (frontend->ops->s_input)
                {
                    ret=frontend->ops->s_input(frontend, index);
                    
                }
                frontend->cur_input=index;
            }
            return ret;
        }
       
    }
    return -1;
}
int  frontend_g_input(IFrontend *frontend)
{
    if(frontend)
    { 
        return frontend->cur_input;
        
    }
    return -1;
    
}

const frontend_frameinfo_t * frontend_g_frameinfo(IFrontend *frontend)
{
    if(frontend)
    { 
        frontend_input_status_e input_status=frontend_g_input_status(frontend);
        if(input_status==FRONTEND_LOCK)
            return &frontend->input_frameinfo;
    }
    return NULL;
}

const frontend_audioinfo_t *frontend_g_audioinfo(IFrontend *frontend)
{
    if(frontend)
    { 
        frontend_input_status_e input_status=frontend_g_input_status(frontend);
        if(input_status==FRONTEND_LOCK)
            return &frontend->audioinfo;
    }
    return NULL;
}
    
//void  frontend_s_out_framesize(IFrontend *frontend,frontend_framesize_t *framesize)
//{
//    if(frontend)
//    {
//        frontend->out_framesize=*framesize;
//    }
//    
//}
//frontend_framesize_t frontend_g_out_framesize(IFrontend *frontend)
//{
//    frontend_framesize_t framesize={0,0};
//    if(frontend)
//    {
//        framesize=frontend->out_framesize;
//    }
//    return framesize;
//}



TQP_BOOL frontend_is_streaming(IFrontend *frontend)
{
    return frontend->streaming;
}

void frontend_s_pixfmt(IFrontend *frontend,u32 pixfmt)
{
    if(frontend)
    {
        frontend->pix_fmt=pixfmt;
    }
}

u32 frontend_g_pixfmt(IFrontend *frontend)
{
    if(frontend)
        return frontend->pix_fmt;
    return 0;
}

void frontend_s_framerate(IFrontend *frontend,int framerate)
{
    if(frontend)
    {
        frontend->out_framerate=framerate;
    }
    
}

int frontend_g_framerate(IFrontend *frontend)
{
    if(frontend)
    {
        return frontend->out_framerate;
    }
    return 0;        
}

const frontend_input_info_t *frontend_g_input_info(IFrontend *frontend,int index)
{
   const frontend_input_info_t *input_info=NULL;
   if(frontend)
   {
        if(index < frontend->input_count)
        {
            input_info=&frontend->input_info[index];
        }
   }
   return input_info;
}

frontend_input_status_e frontend_g_input_status(IFrontend *frontend)
{
    frontend_input_status_e status=FRONTEND_NO_SIGNAL;
    if(frontend)
    {
        status=frontend->input_info[frontend->cur_input].status;
    }    
    return status;
}


void frontend_s_input_status(IFrontend *frontend,int index ,frontend_input_status_e status)
{
    if(frontend)
    {
        if(index <frontend->input_count)
        {
            frontend->input_info[index].status=status;
        }
    }
}    
