#ifndef __V4L2OPS_H__
#define __V4L2OPS_H__

struct pl330_frame;
struct pl330_format;

typedef enum
{
    FRONTEND_EVENT_NO_SIGNAL=0,
    FRONTEND_EVENT_NO_SYNC,
    FRONTEND_EVENT_LOCK,
    FRONTEND_EVENT_READY_STREAMING,
    FRONTEND_EVENT_NUM,
}IFrontend_event_e;

typedef enum
{
    H_POS_V_POS,
    H_POS_V_NEG,
    H_NEG_V_POS,
    H_NEG_V_NEG,
}Frontend_polarity_e;

typedef enum
{
    adv7604_f,
    gv7601_f,
}Frontend_select_e;

typedef struct
{
    short width;
    short height;
    TQP_BOOL is_interlace;
    int framerate;
    Frontend_polarity_e  polarities;
    Frontend_select_e frontend_select;
    
}frontend_frameinfo_t;

typedef struct
{
    int sample_rate;
}frontend_audioinfo_t;

typedef struct
{
    int width;
    int height;
}frontend_framesize_t;


typedef void (*frontend_event_cb_t)(void *cxt,IFrontend_event_e event);
typedef struct
{
    /* for V4L2 Ctrl Handling */
    int (*s_ctrl)(void *frontend, struct v4l2_control *ctrl);
    int (*g_ctrl)(void *frontend, struct v4l2_control *ctrl);

	/* for V4L2 Input Handling */
    int (*s_input)(void *frontend, int index); 
    //int (*g_input)(void *frontend, int *index);

	/* for V4L2 Format Handling */
    int (*try_frame)(void *frontend, struct pl330_frame *frame,	struct pl330_format *format);
    void (*get_frameinfo)(void *frontend,frontend_frameinfo_t *frameinfo);
    void (*get_audioinfo)(void *frontend,frontend_audioinfo_t *audioinfo);
}frontend_ops_t;

enum
{
    FRONTEND_MAX_INPUT=16,
};

typedef enum
{
    FRONTEND_NO_SIGNAL,
    FRONTEND_NO_SYNC,
    FRONTEND_LOCK,
            
}frontend_input_status_e;

typedef struct
{
    const char *name;
    frontend_input_status_e status;
}frontend_input_info_t;


typedef struct 
{
        frontend_ops_t *ops;
        frontend_event_cb_t  event_cb_func;
        const struct v4l2_queryctrl * v4l2_queryctrls;
        int v4l2_queryctrls_num;
        void *cb_cxt;
        TQP_BOOL streaming;
        TQP_BOOL is_lock;
        frontend_frameinfo_t input_frameinfo;
        frontend_audioinfo_t audioinfo;
        frontend_framesize_t out_framesize;
        int out_framerate;
        unsigned int pix_fmt;
        int input_count;
        frontend_input_info_t input_info[FRONTEND_MAX_INPUT];
        int cur_input;
}IFrontend;

typedef struct
{
    int input_count;
    const char *input_name[];
}frontend_setup_t;



void frontend_init(IFrontend *frontend,frontend_setup_t *setup_info);
void frontend_register_event_callback(IFrontend *frontend,frontend_event_cb_t cb_func,void *cb_cxt);
void frontend_update_event(IFrontend *frontend,IFrontend_event_e event);
void frontend_start_streaming(IFrontend *frontend,TQP_BOOL start);
int  frontend_s_ctrl(IFrontend *frontend, struct v4l2_control *ctrl);
int  frontend_g_ctrl(IFrontend *frontend, struct v4l2_control *ctrl);
int  frontend_s_input(IFrontend *frontend, int index); 
int  frontend_g_input(IFrontend *frontend); 
const frontend_input_info_t *frontend_g_input_info(IFrontend *frontend,int index);
frontend_input_status_e frontend_g_input_status(IFrontend *frontend);
void frontend_s_input_status(IFrontend *frontend,int index ,frontend_input_status_e status);
const frontend_frameinfo_t *frontend_g_frameinfo(IFrontend *frontend);
const frontend_audioinfo_t *frontend_g_audioinfo(IFrontend *frontend);
//void  frontend_s_out_framesize(IFrontend *frontend,frontend_framesize_t *framesize);
//frontend_framesize_t frontend_g_out_framesize(IFrontend *frontend);

TQP_BOOL frontend_is_streaming(IFrontend *frontend);
void frontend_s_pixfmt(IFrontend *frontend,unsigned int pixfmt);
unsigned int frontend_g_pixfmt(IFrontend *frontend);
void frontend_s_framerate(IFrontend *frontend,int framerate);
int frontend_g_framerate(IFrontend *frontend);


#endif
