/*************************************************************************/
/** This file and the information contain herein is provided "AS IS"    **/
/** without warranty of any kind, either expressed or implied,          **/
/** including but not limited to the implied warranties of              **/
/** merchantability and/or fitness for a particular purpose.            **/
/**                                                                     **/
/** This code cannot be redistributed or reused in any form whatsoever  **/
/** without the explicit approval of  Sigma Designs, Inc.               **/
/**                                                                     **/
/** Copyright (C) 2011 Sigma Designs, Inc.                              **/
/** All rights reserved.                                                **/
/*************************************************************************/

#include "qlautoconf.h"

#include <linux/errno.h>
#include <linux/init.h>
#include <linux/mm.h>
#include <linux/module.h>
#include <linux/poll.h>
#include <linux/slab.h>
#include <linux/vmalloc.h>
#include <asm/io.h>
#include <linux/string.h>

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/fs.h>
#include <asm/uaccess.h>
#include <asm/fcntl.h>


int qlf_open(unsigned long *pdwHandle, unsigned char *pFName, unsigned int dwReadOnly) 
{
    int Err= 0;
    if(dwReadOnly) {
        *pdwHandle = (unsigned long)filp_open(pFName, 0, 0); 
    } else {
        *pdwHandle = (unsigned long)filp_open(pFName, O_CREAT | O_RDWR | O_TRUNC, 0); 
    }
    if(IS_ERR((void*)*pdwHandle)) {
        Err= *pdwHandle;
        *pdwHandle= 0;
    } 
    printk("qlusb: open fx2fw %s. Err=%d \n", pFName, Err);
    return Err;
}

int qlf_close(unsigned long dwHandle) 
{
    if((NULL != (struct file*)dwHandle) && (!(IS_ERR((struct file*)dwHandle))))
        return filp_close((struct file*)dwHandle, 0);
    else 
        return 0; 
}

int qlf_write(unsigned long dwHandle, unsigned char *pBuff, unsigned int dwLen) 
{
	struct file *file=(struct file *)dwHandle;
	loff_t pos=0;
	unsigned int size,remain=dwLen;

	if(!file)
		return -ENOENT;
	do
	{
		size=kernel_write(file,&pBuff[pos],remain,pos);
		if(size<0)
			break;
		pos+=size;
		remain-=size;
	}while(remain);
	return (size >=0) ? dwLen: size;

}

int qlf_read(unsigned long dwHandle, unsigned char *pBuff, unsigned int dwLen) 
{
	struct file *file=(struct file *)dwHandle;
	loff_t pos=0;
	unsigned int size,remain=dwLen;

	if(!file)
		return -ENOENT;
	do
	{
		size=kernel_read(file,pos,&pBuff[pos],remain);
		if(size<0)
			break;
		pos+=size;
		remain-=size;
	}while(remain);
	return (size >=0) ? dwLen: size;
}



int qlf_load(char *pszFileName, 
			  unsigned char **ppucaBuff, 
			  unsigned int *pulBuffLen
			  )
{
	struct file		*filp = 0;
	int				l=0, curRead;
    int				lErr = 0;
    unsigned char	*pucaBuff = NULL;

	*ppucaBuff = NULL;
	*pulBuffLen = 0;

	lErr = qlf_open((unsigned long *)&filp, 
					pszFileName, 
					1
					);

    if (!lErr && 
		filp
		) 
	{
		l = filp->f_path.dentry->d_inode->i_size;

		// allocate buffer
#ifdef USE_KMALLOC_INSTEAD_OF_VMALLOC
		pucaBuff = (unsigned char *)kmalloc(l, GFP_KERNEL);
#else		
		pucaBuff = (unsigned char *)vmalloc(l);
#endif		

		if (pucaBuff)
		{
			curRead = qlf_read((unsigned long)filp, pucaBuff, l);

			if ((curRead < 0) || 
				(curRead != l)
				)
			{
				printk("Failed to read '%s' expected(%d) read(%d).\n", pszFileName, l, curRead);
#ifdef USE_KMALLOC_INSTEAD_OF_VMALLOC
				kfree(pucaBuff);
#else				
				vfree(pucaBuff);
#endif				
				lErr = -EINVAL;
			}
		}
		else
		{
			printk("Failed to allocate (%d) bytes.\n", l);
			lErr = -ENOMEM;
		}
		qlf_close((unsigned long)filp);
	}

	if (!lErr)
	{
		*ppucaBuff = pucaBuff;
		*pulBuffLen = l;
	}

	printk("%s()-> file %s, size %d\n", __FUNCTION__, pszFileName, *pulBuffLen);

	return (lErr);
}






