///////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2011 Sigma Designs, Inc.  All Rights Reserved.   
//
//		qpsos.h
//
// History:
//	Mar 26, 2008	(EC) Created
//
// Authors: (EC) Enrico Cadorin
//
///////////////////////////////////////////////////////////////////////////////

#ifndef __QPSOS_H__
#define __QPSOS_H__

#include "qptypes.h"


//
// Versioning interface
//

typedef enum {
	QPSOS_MOD_OS_TAG = 0x534F5351,
	QPSOS_MOD_CODECLIB_TAG = 0x4C434443,
	QPSOS_MOD_AUDIO_TAG = 0x49445541,
	QPSOS_MOD_USBLIB_TAG = 0x4C425355,
	QPSOS_MOD_USBLOLIB_TAG = 0x53554F4C,
	QPSOS_MOD_FATLIB_TAG = 0x4C544146,
	QPSOS_MOD_SDIOLIB_TAG = 0x4F494453,
	QPSOS_MOD_CODECDRV_TAG = 0x44434443,
	QPSOS_MOD_FC3LIB_TAG = 0x4C334346,
	QPSOS_MOD_USBMGR_TAG = 0x4D425355,
	QPSOS_MOD_USBTEST_TAG = 0x54425355,
	QPSOS_MOD_USBDEVICE_TAG = 0x44425355,
} EModuleTags;


//
// Macro to extract the major version of QPSOS.
// startAddr is the address of a local memory buffer that should point to
// the QPSOS version record.  This is usually located at offset 0x100 of 
// the firmware image.  Only the first two words are accessed.
//
#if defined (QP_LINXKER)
#define QPSOSGetOSMajorVersion(startAddr)	\
	((*((PTQP_UINT32) startAddr) == QPSOS_MOD_OS_TAG) ?	\
	(*((PTQP_UINT16) (((long) startAddr) + 6))) : 0)
#else

#if defined(_M_AMD64) || defined(_M_IA64)
#define QPSOSGetOSMajorVersion(startAddr)	\
	((*((PTQP_UINT64) startAddr) == QPSOS_MOD_OS_TAG) ?	\
	(*((PTQP_UINT16) (((__int64) startAddr) + 6))) : 0)
#else
#define QPSOSGetOSMajorVersion(startAddr)	\
	((*((PTQP_UINT32) startAddr) == QPSOS_MOD_OS_TAG) ?	\
	(*((PTQP_UINT16) (((int) startAddr) + 6))) : 0)
#endif

#endif


// Special shared memory area mappings.

// Old method (QPSOS 2)

// This area is no longer guaranteed to be flatly mapped by the memory manager
// since it will have to be relocated when running with Linux.
// It is guaranteed to be at 0x2F0000-2FFFFF (64 kB) in physical memory
// It is used for global storage and for debugging information.
// The first 32kB are available to be used by the QPSOS modules and they are
// statically allocated in chunks of 4kB according to the definitions below.
// The remaing 32 kB are reserved to QPSOS use (stacks, translation tables)
//
// WARNING: These addresses are PHYSICAL and therefore they might need to be
// converted to VIRTUAL before being used.
//
#define PHY_COMMONAREA 0x002F0000	// 64kB (up to 2FFFFF)

#define PHY_QPSOSAREA 0x002F0000	// 4kB (up to 2F0FFF) for QPSOS use

#define PHY_CODECAREA 0x002F1000	// 4kB (up to 2F1FFF) for the Codec's needs

#define PHY_USBAREA 0x002F2000		// 4kB (up to 2F2FFF) USB
#define FLAT_USBAREA PHY_USBAREA	// for backward compatibility

#define PHY_BOOTAREA 0x002F3000		// 4kB (up to 2F3FFF) for boot parameters
									// set by the bootloader or the host driver

#define PHY_SHARED_AREA		0x0F0000	// 64k at the end of the first MegaByte

#define PHY_BOOT_AREA		(PHY_SHARED_AREA + 0x0)		// boot parameters shared with loaders (4kB)
#define PHY_CODEC_AREA		(PHY_SHARED_AREA + 0x1000)	// 4kB for the CODEC library
#define PHY_USB_AREA		(PHY_SHARED_AREA + 0x2000)	// 4kB for the USB module


#endif //__QPSOS_H__

