///////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2011 Sigma Designs, Inc.  All Rights Reserved.   
//
//		OS Manager - Linux Kernel Mode specific code
//
// History:
//	April 16, 2008	
//
// Authors: Catherine Cheng, Enrico Cadorin
//
///////////////////////////////////////////////////////////////////////////////

#include "qlautoconf.h"
#include <linux/version.h>
#include <linux/time.h>
#include <asm/param.h>


#if LINUX_VERSION_CODE >= KERNEL_VERSION(4,11,0)
#include <linux/sched/signal.h>
#else
#include <linux/sched.h>
#endif

#include <asm/div64.h>

#include <asm/delay.h>
#include "../common/qptypes.h"
#include "../common/qpdbgmgr.h"
#include "../common/qptimemgr.h"
#include "qptimemgrint.h"

static wait_queue_head_t wq;


EQP_Errors
TimeInit (void)
{
	EQP_Errors		rv = QPERR_SUCCESS;

	init_waitqueue_head(&wq);

	return (rv);
}


void
TimeTerm (void)
{
}


void
TimeGetCaps (PTQP_TMCaps capsP)
{
    /* TBW */
}


TQPTM_Time
TimeGetSysTimeMilliS (void)
{
	unsigned long long ullMilliS;
	ullMilliS = TimeGetSysTimeMicroS();
	do_div(ullMilliS, 1000);
	return ullMilliS;
}

TQPTM_Time
TimeGetSysTimeMicroS (void)
{
	unsigned long long ullMicroS;
	ullMicroS = TimeGetSysTimeNanoS();
	do_div(ullMicroS, 1000);
	return ullMicroS;
}

TQPTM_Time
TimeGetSysTimeNanoS (void)
{

	struct timespec64 ts;
	u64  t;

	ktime_get_ts64(&ts);
	t = *((u64 *)&(ts.tv_sec));
	t = t*1000000000;
	t += ts.tv_nsec;
#if 0
	struct timeval tv;
	struct timezone tz;
	u64 t;

	//do_gettimeofday64(&tv);
	gettimeofday(&tv, &tz);
	t = tv.tv_sec;
	t = t*1000000;
	t += tv.tv_usec;

	t = t*1000;
#endif
	return t;
}


TQPTM_Time
TimeGetSysTicks (void)
{
	TQPTM_Time t = (TQPTM_Time)HZ;
	return t;
}


EQP_Errors
TimeDelayMilliS (TQP_UINT32 milliS)
{
	EQP_Errors		rv = QPERR_SUCCESS;

	//set_current_state( TASK_INTERRUPTIBLE );
	//schedule_timeout( milliS*HZ/1000 );

#if LINUX_VERSION_CODE < KERNEL_VERSION(3,3,0)	
	sleep_on_timeout (&wq, (HZ * milliS + 999) / 1000);
#else
	wait_event_timeout(wq,true,(HZ * milliS + 999) / 1000);
#endif	

	return (rv);
}

EQP_Errors
TimeDelayMicroS (TQP_UINT32 microS)
{
	EQP_Errors		rv = QPERR_SUCCESS;

	udelay( microS );

	return (rv);
}



