
#include "cdevice.h"

#include "qlautoconf.h"
#include <linux/delay.h>
#include <linux/string.h>
#include "platform.h"
#include "qptypes.h"
#include "cpciecntl.h"
#include "qpdbgmgr.h"
#include "qpmemmgr.h"
#include "qptimemgr.h"
#include "util.h"
#include "qpbusintf.h"
#include "qpmpgcodec.h"
#ifdef CONFIG_QL300_PCIE_EVK
#include "pci_config.h"
#include "qlpciintf.h"
#endif
#include "qplinuxkermap.h"

#include "qpdevcntl.h"
#include "hci.h"

#include "plgpio.h"
#include "cgpio_gspi.h"

#define spidelay(x) do { udelay(0); } while(0)

static inline void setsck(struct gpio_gspi_mgr *mgr, int is_on)
{
	write_gpio(mgr->hci, mgr->sck, is_on);
}

static inline void setmosi(struct gpio_gspi_mgr *mgr, int is_on)
{
	write_gpio(mgr->hci, mgr->mosi, is_on ? 1 : 0);
}

static inline int getmiso(struct gpio_gspi_mgr *mgr)
{
	return read_gpio(mgr->hci, mgr->miso);
}

static inline void start(struct gpio_gspi_mgr *mgr)
{
	write_gpio(mgr->hci, mgr->chipselect, 0);
}

static inline void stop(struct gpio_gspi_mgr *mgr)
{
	setmosi(mgr, 0);
	write_gpio(mgr->hci, mgr->chipselect, 1);
}

static inline void gspi_init(struct gpio_gspi_mgr *mgr)
{
}

void __gpio_gspi_write(struct gpio_gspi_mgr *mgr, u16 data)
{
	u16 bits = 0x8000;
	for(; bits; bits >>= 1) {
		setmosi(mgr, data & bits);
//		spidelay(3);

		setsck(mgr, 1);

//		spidelay(3);
		setsck(mgr, 0);
	}
}

void __gpio_gspi_read(struct gpio_gspi_mgr *mgr, u16 *data)
{
	u16 bits = 0x8000;
	u16 val = 0;

	setmosi(mgr, 0);
	for(; bits; bits >>= 1) {
		setsck(mgr, 1);

		if(getmiso(mgr))
			val |= bits;

		setsck(mgr, 0);
	}

	*data = val;
}

void gpio_gspi_write(struct gpio_gspi_mgr *mgr, u16 command, u16 data)
{
	mutex_lock(&mgr->mlock);
	stop(mgr);
	start(mgr);
	__gpio_gspi_write(mgr,  command);
	__gpio_gspi_write(mgr,  data);
	stop(mgr);
	mutex_unlock(&mgr->mlock);
}


void gpio_gspi_read(struct gpio_gspi_mgr *mgr, u16 command, u16 *data)
{
	mutex_lock(&mgr->mlock);
	stop(mgr);
	start(mgr);
	__gpio_gspi_write(mgr,  command);
//	spidelay(4);
	__gpio_gspi_read(mgr,  data);
	stop(mgr);
	mutex_unlock(&mgr->mlock);
}

void gpio_gspi_init(struct gpio_gspi_mgr *mgr, IHCIAPI *hci, int miso, int mosi, int sck)
{
	printk("%s\n", __func__);

	mutex_init(&mgr->mlock);
	mgr->hci = hci;
	mgr->miso = miso;
    //LOG_NUM(miso);
	mgr->mosi = mosi;
    //LOG_NUM(mosi);
	mgr->sck = sck;
    //LOG_NUM(sck);

	/* MISO set by slave */
	set_gpio_dir(hci, miso, GPIO_INPUT);
	stop(mgr);
}

void gpio_gspi_release(struct gpio_gspi_mgr *mgr)
{
}


void gpio_gspi_chipselect(struct gpio_gspi_mgr *mgr, unsigned int chip)
{
	mgr->chipselect = chip;
	write_gpio(mgr->hci, chip, 1);
}
