#ifndef __CGPIO_I2C_H__
#define __CGPIO_I2C_H__
#include "platform.h"
/*
 * I2C function using GPIO
 *
 */
#define REG_BASE				0x80000000
//#define REG_BASE				0x00000000
#define GPIO_CONTROL_DIR			(REG_BASE+0x610)
#define GPIO_WRITE_PORT				(REG_BASE+0x614)
#define GPIO_READ_PORT				(REG_BASE+0x618)
#define GPIO_INTERRUPT				(REG_BASE+0x61C)

#ifndef PLGPIO_PIN
#define PLGPIO_PIN
enum GPIO_PIN {
	GPIO_00 = 0,
	GPIO_01,
	GPIO_02,
	GPIO_03,
	GPIO_04,
	GPIO_05,
	GPIO_06,
	GPIO_07,
	GPIO_08,
	GPIO_09,
	GPIO_10,
	GPIO_11,
	GPIO_12,
	GPIO_13,
	GPIO_14,
	GPIO_15,
};
#endif

/**
 * struct i2c_msg - an I2C transaction segment beginning with START
 * @addr: Slave address, either seven or ten bits.  When this is a ten
 *	bit address, I2C_M_TEN must be set in @flags and the adapter
 *	must support I2C_FUNC_10BIT_ADDR.
 * @flags: I2C_M_RD is handled by all adapters.  No other flags may be
 *	provided unless the adapter exported the relevant I2C_FUNC_*
 *	flags through i2c_check_functionality().
 * @len: Number of data bytes in @buf being read from or written to the
 *	I2C slave address.  For read transactions where I2C_M_RECV_LEN
 *	is set, the caller guarantees that this buffer can hold up to
 *	32 bytes in addition to the initial length byte sent by the
 *	slave (plus, if used, the SMBus PEC); and this value will be
 *	incremented by the number of block data bytes received.
 * @buf: The buffer into which data is read, or from which it's written.
 *
 * An i2c_msg is the low level representation of one segment of an I2C
 * transaction.  It is visible to drivers in the @i2c_transfer() procedure,
 * to userspace from i2c-dev, and to I2C adapter drivers through the
 * @i2c_adapter.@master_xfer() method.
 *
 * Except when I2C "protocol mangling" is used, all I2C adapters implement
 * the standard rules for I2C transactions.  Each transaction begins with a
 * START.  That is followed by the slave address, and a bit encoding read
 * versus write.  Then follow all the data bytes, possibly including a byte
 * with SMBus PEC.  The transfer terminates with a NAK, or when all those
 * bytes have been transferred and ACKed.  If this is the last message in a
 * group, it is followed by a STOP.  Otherwise it is followed by the next
 * @i2c_msg transaction segment, beginning with a (repeated) START.
 *
 * Alternatively, when the adapter supports I2C_FUNC_PROTOCOL_MANGLING then
 * passing certain @flags may have changed those standard protocol behaviors.
 * Those flags are only for use with broken/nonconforming slaves, and with
 * adapters which are known to support the specific mangling options they
 * need (one or more of IGNORE_NAK, NO_RD_ACK, NOSTART, and REV_DIR_ADDR).
 */
#define I2C_M_WRITE		0x0000
#define I2C_M_TEN		0x0010	/* this is a ten bit chip address */
#define I2C_M_RD		0x0001	/* read data, from slave to master */
#define I2C_M_NOSTART		0x4000	/* if I2C_FUNC_PROTOCOL_MANGLING */
#define I2C_M_REV_DIR_ADDR	0x2000	/* if I2C_FUNC_PROTOCOL_MANGLING */
#define I2C_M_IGNORE_NAK	0x1000	/* if I2C_FUNC_PROTOCOL_MANGLING */
#define I2C_M_NO_RD_ACK		0x0800	/* if I2C_FUNC_PROTOCOL_MANGLING */
#define I2C_M_RECV_LEN		0x0400	/* length will be first received byte */

struct aver_i2c_msg {
	TQP_UINT16 addr;	/* slave address			*/
	TQP_UINT16 flags;
	TQP_UINT16 len;		/* msg length				*/
	TQP_UINT8 *buf;		/* pointer to msg data			*/
};

#define I2C_SMBUS_BLOCK_MAX	32	/* As specified in SMBus standard */
//extern int bit_xfer(struct i2c_msg* pmsg, int nr_msgs);


#ifndef _QP_QPSOS
#include "hci.h"
#include "qpi2c.h"
#endif //!_QP_QPSOS

struct gpio_i2c_device {
	enum GPIO_PIN	clk;
	enum GPIO_PIN	data;
	TQPTM_Time		udelay;
	TQPTM_Time		utimeout;
	unsigned		is_inited;
	TQP_OSMSemID	i2c_mutex;
#ifndef _QP_QPSOS
	IHCIAPI	*pHCI;
#endif //!_QP_QPSOS
};

#ifdef _QP_QPSOS
#define _GPIO_READ_REGISTER(This, addr, value) \
	value = QPSOSRegRead(addr)
#define _GPIO_WRITE_REGISTER(This, addr, value) \
	QPSOSRegWrite(addr, value)
#define _IHCIAPI_PTR_COMMA	
#define _II2C_PTR_COMMA 
#define _II2C_PTR II2C 
#define _II2C_PTR_OR_VOID void
#define _II2C_PTR_ENT 
#define _II2C_PTR_ENT_COMMA 
#else //!_QP_QPSOS
#define _GPIO_READ_REGISTER(This, addr, value) \
	QPHCI_RegisterRead(This->pHCI, (TQP_UINT16)addr, &value)
#define _GPIO_WRITE_REGISTER(This, addr, value) \
	QPHCI_RegisterWrite(This->pHCI, (TQP_UINT16)addr, value)
#define _IHCIAPI_PTR_COMMA IHCIAPI *pHCI,
#define _II2C_PTR_COMMA II2C *pI2C,
#define _II2C_PTR II2C *pI2C
#define _II2C_PTR_OR_VOID II2C *pI2C
#define _II2C_PTR_ENT pI2C
#define _II2C_PTR_ENT_COMMA pI2C,
#endif //_QP_QPSOS

int i2c_init(
	_II2C_PTR_COMMA
	_IHCIAPI_PTR_COMMA
	enum GPIO_PIN clk, 
	enum GPIO_PIN data, 
	unsigned udelay, 
	unsigned utimeout
	);
	
void i2c_term(
	_II2C_PTR_OR_VOID
	);

/* Not to be called in_interrupt */

int aver_i2c_transfer(
	_II2C_PTR_COMMA
	struct aver_i2c_msg* pmsg, 
	int nr_msgs
	);


#endif /*__CGPIO_I2C_H__*/
