///////////////////////////////////////////////////////////////////////////////
//
// cadv7604.c
//
// Description: 
//   ADV7604 implementation
// 
// History:
//	 Aug 15, 2013 created
//
// Authors: Shihpo Wang
//
///////////////////////////////////////////////////////////////////////////////


#ifndef __CGV7601_H__
#define __CGV7601_H__

#include <linux/videodev2.h>
#include <linux/version.h>
#include <media/v4l2-common.h>
#include <media/v4l2-device.h>
#include <media/v4l2-ctrls.h>

#if (LINUX_VERSION_CODE < KERNEL_VERSION(3,7,0))
#include "v4l2-dv-timings.h"
#else
#include <media/v4l2-common.h>
#include <linux/v4l2-dv-timings.h>
#endif

#include "cviddecoder.h"
#include "qpi2c.h"

#include "cm313b.h"
#include "v4l2ops.h"
#include "pl330.h"
#include "cgpio_gspi.h"
#include "boardinfo-probe.h"

struct pl330_frame;
#ifdef __cplusplus
extern "C" {
#endif

struct pl330_frame;
struct pl330_format;

struct pl330_v4l2_ops
{
	/* for V4L2 Ctrl Handling */
	int (*s_ctrl)(void *viu, struct v4l2_control *ctrl);
	int (*g_ctrl)(void *viu, struct v4l2_control *ctrl);

	/* for V4L2 Input Handling */
	int (*s_input)(void *viu, int index); 
	int (*g_input)(void *viu, int *index);

	/* for V4L2 Format Handling */
	int (*try_frame)(void *viu, struct pl330_frame *frame,
			struct pl330_format *format);
};


/////////////////////////////////////////////////////////////////////////////
struct gv7601_state {
	struct v4l2_dv_timings timings;

	struct pl330_frame cur_out_frame;
	struct pl330_format format;
	u32 frame_rate;
    u32 out_frame_rate;
	u32 bit_rate;

	bool is_lock;
	bool is_std_lock;
	bool interlaced; 
	bool last_interlaced; 
	int ceaid;
	int vd_std;
	int now_width;
	int now_height;
	int last_width;
	int last_height;
};

struct gv7601_fmt_change
{
	unsigned int		fmt_change;
	struct list_head	queue_head;

    struct v4l2_event   event;
};

typedef struct CGV7601
{
	// public CVideoDecoder
	//
	CVideoDecoder	m_VideoDecoder;

	// CADV7604
	II2C			*m_pI2C;				// I2C interface
	TQP_UINT8		m_cI2CDevAddr;			// i2c device address

	struct gv7601_state state;
	enum cm313b_input_type input;
	struct gpio_gspi_mgr gspi_mgr;

	struct mutex				fmt_lock;
	struct task_struct			*mon_thread;
	struct list_head			queue_head;
	unsigned int				queue_size;

	/* V4L2 ops */
	struct pl330_v4l2_ops		v4l2_ops;

    int (*g_input_status)(void *This, u32 *status);

} CGV7601;


EQP_Errors CGV7601_Create(CQLCodecLib* This ,Chip_Cfg * chip_cfg);
void CGV7601_Delete(CQLCodecLib* This);
CGV7601 * CGV7601_Constructor(CGV7601 *pGV7601, 
							    CObject *pParent,
							    TQP_UINT32 dwAttributes,
							    II2C *pI2C,
							    TQP_UINT8 cI2CDevAddr
							    );
void CGV7601_Destructor(CGV7601 *This);

#ifdef __cplusplus
}
#endif

#endif //__CGV7601_H__


