#include "pl330-dbg.h"
#include "pl330.h"
#include "qpdevcntl.h"
#if (LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,38))
#include "kstrtox.h"
#endif

QPCODEC_SYSTEM_CONTROL			__sys_ctrl;
QPCODEC_PICTURE_RESOLUTION		__pic_res;
QPCODEC_INPUT_CONTROL			__input_ctrl;
QPCODEC_RATE_CONTROL			__rate_ctrl;
QPCODEC_VBR_BIT_RATE			__vbr_ctrl;
QPCODEC_FILTER_CONTROL			__filter_ctrl;
QPCODEC_GOP_LOOP_FILTER			__glf_ctrl;
QPCODEC_OUT_PICTURE_RESOLUTION	__pic_out_res;
QPCODEC_AUDIO_CONTROL_PARAM		__aud_ctrl;
QPCODEC_AUDIO_CONTROL_EX		__aud_ctrl_ex;
QPCODEC_SYS_FUNCTION			__sys_func;
QPCODEC_SYS_LINK				__sys_link;

TQP_UINT32						__deint_mode;
TQP_UINT32						__jpeg_qual;
TQP_UINT32						__jpeg_buf_nr;


#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,32))
#ifndef kstrtol
static inline int __must_check kstrtol(const char *s, unsigned int base, long *res)
{
	if(sizeof(long) == sizeof(long long) &&
	  __alignof__(long) == __alignof__(long long))
		return kstrtoll(s, base, (long long *)res);
	else
		return _kstrtol(s, base, res);
}
#endif
#endif

struct config_item
{
	TQP_UINT32 val;
	char *desc;
};

struct qpcodec_item_config_table
{
	char *item_name;

	int set_ctrl_id;

	int get_ctrl_id;

	/* qpcodec control struct */
	void *qpcodec_ctrl;

	/* item offset*/
	int offset;

	/* configuration description */
	struct config_item item[32];
};


struct qpcodec_item_config_table pl330_config_table[] =
{
	{
		"SystemControl-StreamType", PLDEV_CNTL_IOCTL_SET_SYSTEM_CONTROL,
		 PLDEV_CNTL_IOCTL_GET_SYSTEM_CONTROL,
		&__sys_ctrl, FIELD_OFFSET(QPCODEC_SYSTEM_CONTROL, Artesa.stream_type),
		{
			{0, "ps"}, {1, "ts"}, {2, "mp4"}, {3, "es"}, {4, "raw"},
			{5, "special1"}, {6, "jpeg"},
			{0, 0}
		}
	},
	{
		"SystemControl-StreamData", PLDEV_CNTL_IOCTL_SET_SYSTEM_CONTROL,
		 PLDEV_CNTL_IOCTL_GET_SYSTEM_CONTROL,
		&__sys_ctrl, FIELD_OFFSET(QPCODEC_SYSTEM_CONTROL, Artesa.stream_data),
		{
			{1, "video"}, {2, "audio"}, {3, "video+audio"}, {4, "vbi"}, {8, "user"},
			{0, 0}
		}
	},
	{
		"SystemControl-Profile", PLDEV_CNTL_IOCTL_SET_SYSTEM_CONTROL,
		 PLDEV_CNTL_IOCTL_GET_SYSTEM_CONTROL,
		&__sys_ctrl, FIELD_OFFSET(QPCODEC_SYSTEM_CONTROL, Artesa.profile),
		{
			{0, "baseline"}, {1, "main"}, {2, "extended"},
			{0, 0}
		}
	},
	{
		"SystemControl-Level", PLDEV_CNTL_IOCTL_SET_SYSTEM_CONTROL,
		 PLDEV_CNTL_IOCTL_GET_SYSTEM_CONTROL,
		&__sys_ctrl, FIELD_OFFSET(QPCODEC_SYSTEM_CONTROL, Artesa.level),
		{
			{0, "1"}, {1, "1.b"}, {2, "1.1"}, {3, "1.2"}, {4, "1.3"}, {5, "2"},
			{6, "2.1"}, {7, "2.2"}, {8, "3"}, {9, "3.1"}, {10, "3.2"}, {11, "4"},
			{12, "4.1"}, {13, "4.2"}, {14, "5"}, {15, "5.1"},
			{0, 0}
		}
	},
	{
		"SystemControl-FFMode", PLDEV_CNTL_IOCTL_SET_SYSTEM_CONTROL,
		 PLDEV_CNTL_IOCTL_GET_SYSTEM_CONTROL,
		&__sys_ctrl, FIELD_OFFSET(QPCODEC_SYSTEM_CONTROL, Artesa.ff_mode),
		{
			{0, "auto"}, {1, "frame"}, {2, "field"},
			{0, 0}
		}
	},
	{
		"SystemControl-SpsrFreq", PLDEV_CNTL_IOCTL_SET_SYSTEM_CONTROL,
		 PLDEV_CNTL_IOCTL_GET_SYSTEM_CONTROL,
		&__sys_ctrl, FIELD_OFFSET(QPCODEC_SYSTEM_CONTROL, Artesa.spsr_freq),
		{
			{0, 0}
		}
	},
	{
		"SystemControl-VMode", PLDEV_CNTL_IOCTL_SET_SYSTEM_CONTROL,
		 PLDEV_CNTL_IOCTL_GET_SYSTEM_CONTROL,
		&__sys_ctrl, FIELD_OFFSET(QPCODEC_SYSTEM_CONTROL, Artesa.v_mode),
		{
			{0, "cavlc"}, {1, "cabac"},
			{0, 0}
		}
	},
	{
		"SystemControl-CabacInitIdc", PLDEV_CNTL_IOCTL_SET_SYSTEM_CONTROL,
		 PLDEV_CNTL_IOCTL_GET_SYSTEM_CONTROL,
		&__sys_ctrl, FIELD_OFFSET(QPCODEC_SYSTEM_CONTROL, Artesa.cabac_init_idc),
		{
			{0, 0}
		}
	},
	/* Version */
	{
		"SystemControl-Version", PLDEV_CNTL_IOCTL_SET_SYSTEM_CONTROL,
		 PLDEV_CNTL_IOCTL_GET_SYSTEM_CONTROL,
		&__sys_ctrl, 0,
		{
			{0, 0}
		}
	},
	/* Xfer-Mode */
	{
		"SystemControl-XferMode", PLDEV_CNTL_IOCTL_SET_SYSTEM_CONTROL,
		 PLDEV_CNTL_IOCTL_GET_SYSTEM_CONTROL,
		&__sys_ctrl, 0,
		{
			{0, "frame"}, {1, "block"},
			{0, 0}
		}
	},
	/* PictureResolution */
	{
		"PictureResolution-InPicWidth", PLDEV_CNTL_IOCTL_SET_PICTURE_RESOLUTION,
		PLDEV_CNTL_IOCTL_GET_PICTURE_RESOLUTION,
		&__pic_res, FIELD_OFFSET(QPCODEC_PICTURE_RESOLUTION, Artesa.in_pic_width),
		{
			{0, 0},
		}
	},
	{
		"PictureResolution-InPicHeight", PLDEV_CNTL_IOCTL_SET_PICTURE_RESOLUTION,
		PLDEV_CNTL_IOCTL_GET_PICTURE_RESOLUTION,
		&__pic_res, FIELD_OFFSET(QPCODEC_PICTURE_RESOLUTION, Artesa.in_pic_height),
		{
			{0, 0},
		}
	},
	/* InputControl */
	{
		"InputControl-ScanFormat", PLDEV_CNTL_IOCTL_SET_INPUT_CONTROL,
		PLDEV_CNTL_IOCTL_GET_INPUT_CONTROL,
		&__input_ctrl, FIELD_OFFSET(QPCODEC_INPUT_CONTROL, Artesa.scan_format),
		{
			{0, "interlace"}, {1, "progressive"}, {2, "sepcial1"}, {3, "special2"}, {4, "special3"},
			{0, 0}
		}
	},
	{
		"InputControl-SrcMode", PLDEV_CNTL_IOCTL_SET_INPUT_CONTROL,
		PLDEV_CNTL_IOCTL_GET_INPUT_CONTROL,
		&__input_ctrl, FIELD_OFFSET(QPCODEC_INPUT_CONTROL, Artesa.src_mode),
		{
			{0, "656"}, {1, "hdmi"}, {2, "hiu"}, {3, "hdimx2"},
			{0, 0}
		}
	},
	{
		"InputControl-SyncMode", PLDEV_CNTL_IOCTL_SET_INPUT_CONTROL,
		PLDEV_CNTL_IOCTL_GET_INPUT_CONTROL,
		&__input_ctrl, FIELD_OFFSET(QPCODEC_INPUT_CONTROL, Artesa.sync_mode),
		{
			{0, "1920x1080i"}, {1, "1920x1080p"}, {2, "1440x1088i"}, {3, "1440x576p"},
			{4, "1440x480p"}, {5, "1280x720p"}, {6, "720_1440x480i"}, {7, "720_1440x576i"},
			{8, "720x480p"}, {9, "704x576p"}, {10, "720x480i"}, {11, "704x576i"},
			{12, "640x480p"}, {13, "640x480i"}, {14, "176x188i"},
			{15, "2"}, {16, "3"}, {17, "0"}, {18, "1"},
			{0, 0}
		}
	},
	{
		"InputControl-DataType", PLDEV_CNTL_IOCTL_SET_INPUT_CONTROL,
		PLDEV_CNTL_IOCTL_GET_INPUT_CONTROL,
		&__input_ctrl, FIELD_OFFSET(QPCODEC_INPUT_CONTROL, Artesa.data_type),
		{
			{0, "h264"}, {1, "mpeg2"}, {2, "jpeg"}, {3, "raw"},
			{0, 0}
		}
	},
	{
		"InputControl-InFrameRate", PLDEV_CNTL_IOCTL_SET_INPUT_CONTROL,
		PLDEV_CNTL_IOCTL_GET_INPUT_CONTROL,
		&__input_ctrl, FIELD_OFFSET(QPCODEC_INPUT_CONTROL, Artesa.in_frame_rate),
		{
			{0, 0}
		}
	},
	{
		"InputControl-OutFrameRate", PLDEV_CNTL_IOCTL_SET_INPUT_CONTROL,
		PLDEV_CNTL_IOCTL_GET_INPUT_CONTROL,
		&__input_ctrl, FIELD_OFFSET(QPCODEC_INPUT_CONTROL, Artesa.out_frame_rate),
		{
			{0, 0}
		}
	},
	{
		"InputControl-Fmt", PLDEV_CNTL_IOCTL_SET_INPUT_CONTROL,
		PLDEV_CNTL_IOCTL_GET_INPUT_CONTROL,
		&__input_ctrl, FIELD_OFFSET(QPCODEC_INPUT_CONTROL, Artesa.fmt),
		{
			{0, "progressive"}, {1, "interlace"},
			{0, 0}
		}
	},
	{
		"InputControl-Force60pIn", PLDEV_CNTL_IOCTL_SET_INPUT_CONTROL,
		PLDEV_CNTL_IOCTL_GET_INPUT_CONTROL,
		&__input_ctrl, FIELD_OFFSET(QPCODEC_INPUT_CONTROL, Artesa.force_60p_in),
		{
			{0, "0"}, {1, "1"},
			{0, 0}
		}
	},
	{
		"InputControl-PdEn", PLDEV_CNTL_IOCTL_SET_INPUT_CONTROL,
		PLDEV_CNTL_IOCTL_GET_INPUT_CONTROL,
		&__input_ctrl, FIELD_OFFSET(QPCODEC_INPUT_CONTROL, Artesa.pd_en),
		{
			{0, 0}
		}
	},
	{
		"InputControl-CkEdge", PLDEV_CNTL_IOCTL_SET_INPUT_CONTROL,
		PLDEV_CNTL_IOCTL_GET_INPUT_CONTROL,
		&__input_ctrl, FIELD_OFFSET(QPCODEC_INPUT_CONTROL, Artesa.ck_edge),
		{
			{0, "positive"}, {1, "negative"},
			{0, 0}
		}
	},
	{
		"InputControl-ByteSwap", PLDEV_CNTL_IOCTL_SET_INPUT_CONTROL,
		PLDEV_CNTL_IOCTL_GET_INPUT_CONTROL,
		&__input_ctrl, FIELD_OFFSET(QPCODEC_INPUT_CONTROL, Artesa.bytes_swap),
		{
			{0, "0"}, {1, "1"},
			{0, 0}
		}
	},
    /* RateControl */
	{
		"RateControl-AvgBitRate", PLDEV_CNTL_IOCTL_SET_RATE_CONTROL,
		PLDEV_CNTL_IOCTL_GET_RATE_CONTROL,
		&__rate_ctrl, FIELD_OFFSET(QPCODEC_RATE_CONTROL, Artesa.avg_bit_rate),
		{
			{0, 0}
		}
	},
	{
		"RateControl-QpUpdateFreq", PLDEV_CNTL_IOCTL_SET_RATE_CONTROL,
		PLDEV_CNTL_IOCTL_GET_RATE_CONTROL,
		&__rate_ctrl, FIELD_OFFSET(QPCODEC_RATE_CONTROL, Artesa.qp_update_freq),
		{
			{0, 0}
		}
	},
	{
		"RateControl-Mode", PLDEV_CNTL_IOCTL_SET_RATE_CONTROL,
		PLDEV_CNTL_IOCTL_GET_RATE_CONTROL,
		&__rate_ctrl, FIELD_OFFSET(QPCODEC_RATE_CONTROL, Artesa.mode),
		{
			{0, "viu"}, {1, "hiu"},
			{0, 0}
		}
	},
	{
		"RateControl-Fixed", PLDEV_CNTL_IOCTL_SET_RATE_CONTROL,
		PLDEV_CNTL_IOCTL_GET_RATE_CONTROL,
		&__rate_ctrl, FIELD_OFFSET(QPCODEC_RATE_CONTROL, Artesa.fixed),
		{
			{0, "normal"}, {1, "fixed"},
			{0, 0}
		}
	},
	{
		"RateControl-Vbr", PLDEV_CNTL_IOCTL_SET_RATE_CONTROL,
		PLDEV_CNTL_IOCTL_GET_RATE_CONTROL,
		&__rate_ctrl, FIELD_OFFSET(QPCODEC_RATE_CONTROL, Artesa.vbr),
		{
			{0, 0}
		}
	},
    /* VbrBitRate */
	{
		"VbrBitRate-MinBitRate", PLDEV_CNTL_IOCTL_SET_VBR_BIT_RATE,
		PLDEV_CNTL_IOCTL_GET_VBR_BIT_RATE,
		&__vbr_ctrl, FIELD_OFFSET(QPCODEC_VBR_BIT_RATE, min_bit_rate),
		{
			{0, 0}
		}
	},
	{
		"VbrBitRate-MaxBitRate", PLDEV_CNTL_IOCTL_SET_VBR_BIT_RATE,
		PLDEV_CNTL_IOCTL_GET_VBR_BIT_RATE,
		&__vbr_ctrl, FIELD_OFFSET(QPCODEC_VBR_BIT_RATE, max_bit_rate),
		{
			{0, 0}
		}
	},
    /* GopLoopFilter */
	{
		"GopLoopFilter-IntraPeriod", PLDEV_CNTL_IOCTL_SET_GOP_LOOP_FILTER,
		PLDEV_CNTL_IOCTL_GET_GOP_LOOP_FILTER,
		&__glf_ctrl, FIELD_OFFSET(QPCODEC_GOP_LOOP_FILTER, Artesa.intra_period),
		{
			{0, 0}
		}
	},
	{
		"GopLoopFilter-IpFilter", PLDEV_CNTL_IOCTL_SET_GOP_LOOP_FILTER,
		PLDEV_CNTL_IOCTL_GET_GOP_LOOP_FILTER,
		&__glf_ctrl, FIELD_OFFSET(QPCODEC_GOP_LOOP_FILTER, Artesa.lp_filter),
		{
			{0, 0}
		}
	},
	{
		"GopLoopFilter-DisIfIdc", PLDEV_CNTL_IOCTL_SET_GOP_LOOP_FILTER,
		PLDEV_CNTL_IOCTL_GET_GOP_LOOP_FILTER,
		&__glf_ctrl, FIELD_OFFSET(QPCODEC_GOP_LOOP_FILTER, Artesa.dis_lf_idc),
		{
			{0, 0}
		}
	},
	{
		"GopLoopFilter-AlphaOffset", PLDEV_CNTL_IOCTL_SET_GOP_LOOP_FILTER,
		PLDEV_CNTL_IOCTL_GET_GOP_LOOP_FILTER,
		&__glf_ctrl, FIELD_OFFSET(QPCODEC_GOP_LOOP_FILTER, Artesa.alpha_offset),
		{
			{0, 0}
		}
	},
	{
		"GopLoopFilter-BetaOffset", PLDEV_CNTL_IOCTL_SET_GOP_LOOP_FILTER,
		PLDEV_CNTL_IOCTL_GET_GOP_LOOP_FILTER,
		&__glf_ctrl, FIELD_OFFSET(QPCODEC_GOP_LOOP_FILTER, Artesa.beta_offset),
		{
			{0, 0}
		}
	},
	{
		"GopLoopFilter-BNum", PLDEV_CNTL_IOCTL_SET_GOP_LOOP_FILTER,
		PLDEV_CNTL_IOCTL_GET_GOP_LOOP_FILTER,
		&__glf_ctrl, FIELD_OFFSET(QPCODEC_GOP_LOOP_FILTER, Artesa.B_num),
		{
			{0, 0}
		}
	},
	{
		"GopLoopFilter-T8x8", PLDEV_CNTL_IOCTL_SET_GOP_LOOP_FILTER,
		PLDEV_CNTL_IOCTL_GET_GOP_LOOP_FILTER,
		&__glf_ctrl, FIELD_OFFSET(QPCODEC_GOP_LOOP_FILTER, Artesa.t8x8),
		{
			{0, 0}
		}
	},
	{
		"GopLoopFilter-RefP", PLDEV_CNTL_IOCTL_SET_GOP_LOOP_FILTER,
		PLDEV_CNTL_IOCTL_GET_GOP_LOOP_FILTER,
		&__glf_ctrl, FIELD_OFFSET(QPCODEC_GOP_LOOP_FILTER, Artesa.ref_p),
		{
			{0, "off"}, {1, "on"},
			{0, 0}
		}
	},
	{
		"GopLoopFilter-RefN", PLDEV_CNTL_IOCTL_SET_GOP_LOOP_FILTER,
		PLDEV_CNTL_IOCTL_GET_GOP_LOOP_FILTER,
		&__glf_ctrl, FIELD_OFFSET(QPCODEC_GOP_LOOP_FILTER, Artesa.ref_n),
		{
			{0, "all"}, {1, "one"},
			{0, 0}
		}
	},
	{
		"GopLoopFilter-Idr", PLDEV_CNTL_IOCTL_SET_GOP_LOOP_FILTER,
		PLDEV_CNTL_IOCTL_GET_GOP_LOOP_FILTER,
		&__glf_ctrl, FIELD_OFFSET(QPCODEC_GOP_LOOP_FILTER, Artesa.IDR),
		{
			{0, "open"}, {1, "close"},
			{0, 0}
		}
	},
    /* FilterControl */
	{
		"FilterControl-StartPixel", PLDEV_CNTL_IOCTL_SET_FILTER_CONTROL,
		PLDEV_CNTL_IOCTL_GET_FILTER_CONTROL,
		&__filter_ctrl, FIELD_OFFSET(QPCODEC_FILTER_CONTROL, start_pixel),
		{
			{0, 0}
		}
	},
	{
		"FilterControl-FlEn", PLDEV_CNTL_IOCTL_SET_FILTER_CONTROL,
		PLDEV_CNTL_IOCTL_GET_FILTER_CONTROL,
		&__filter_ctrl, FIELD_OFFSET(QPCODEC_FILTER_CONTROL, fl_en),
		{
			{0, 0}
		}
	},
	{
		"FilterControl-FilterStrength", PLDEV_CNTL_IOCTL_SET_FILTER_CONTROL,
		PLDEV_CNTL_IOCTL_GET_FILTER_CONTROL,
		&__filter_ctrl, FIELD_OFFSET(QPCODEC_FILTER_CONTROL, filter_strength),
		{
			{0, 0}
		}
	},
	{
		"FilterControl-StartLine", PLDEV_CNTL_IOCTL_SET_FILTER_CONTROL,
		PLDEV_CNTL_IOCTL_GET_FILTER_CONTROL,
		&__filter_ctrl, FIELD_OFFSET(QPCODEC_FILTER_CONTROL, start_line),
		{
			{0, 0}
		}
	},
	{
		"FilterControl-DisSkp", PLDEV_CNTL_IOCTL_SET_FILTER_CONTROL,
		PLDEV_CNTL_IOCTL_GET_FILTER_CONTROL,
		&__filter_ctrl, FIELD_OFFSET(QPCODEC_FILTER_CONTROL, dis_skp),
		{
			{0, 0}
		}
	},
    /* OutPictureResolution */
	{
		"OutPictureResolution-OutPicWidth", PLDEV_CNTL_IOCTL_SET_OUT_PICTURE_RESOLUTION,
		PLDEV_CNTL_IOCTL_GET_OUT_PICTURE_RESOLUTION,
		&__pic_out_res, FIELD_OFFSET(QPCODEC_OUT_PICTURE_RESOLUTION, out_pic_width),
		{
			{0, 0}
		}
	},
	{
		"OutPictureResolution-OutPicHeight", PLDEV_CNTL_IOCTL_SET_OUT_PICTURE_RESOLUTION,
		PLDEV_CNTL_IOCTL_GET_OUT_PICTURE_RESOLUTION,
		&__pic_out_res, FIELD_OFFSET(QPCODEC_OUT_PICTURE_RESOLUTION, out_pic_height),
		{
			{0, 0}
		}
	},
    /* EncMjpegQuality, 0(fw internal control),1 to 15 */
	{
		"EncMJpeg-Quality", PLDEV_CNTL_IOCTL_SET_ENC_MJPEG_QUALITY,
		PLDEV_CNTL_IOCTL_GET_ENC_MJPEG_QUALITY,
		&__jpeg_qual, 0, { {0, 0} }
	},
    /* EncStillVideoInput,  0(fw internal control),1 to 30 */
	{
		"EncMjpeg-FrameBufNb", PLDEV_CNTL_IOCTL_SET_ENC_MJPEG_FRAME_BUF,
		PLDEV_CNTL_IOCTL_GET_ENC_MJPEG_QUALITY,
		&__jpeg_buf_nr, 0, { {0, 0} }
	},

    /* AudioControlParam */
	{
		"AudioControlParam-BitRate", PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_PARAM,
		PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_PARAM,
		&__aud_ctrl, FIELD_OFFSET(QPCODEC_AUDIO_CONTROL_PARAM, bit_rate),
		{
			{0, 0}
		}
	},
	{
		"AudioControlParam-ChSel", PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_PARAM,
		PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_PARAM,
		&__aud_ctrl, FIELD_OFFSET(QPCODEC_AUDIO_CONTROL_PARAM, ch_sel),
		{
			{0, "mux"}, {1, "left"}, {2, "right"},
			{0, 0}
		}
	},
	{
		"AudioControlParam-SampleRate", PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_PARAM,
		PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_PARAM,
		&__aud_ctrl, FIELD_OFFSET(QPCODEC_AUDIO_CONTROL_PARAM, sample_rate),
		{
			{0, "44.1k"}, {1, "48k"}, {2, "32k"}, {4, "22.5k"}, {5, "24k"},
			{6, "16k"}, {8, "8k"}, {9, "88.2k"}, {10, "96k"}, {11, "64k"},
			{12, "176.4k"}, {13, "192k"}, {14, "128k"},
			{0, 0}
		}
	},
	{
		"AudioControlParam-AouPassEn", PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_PARAM,
		PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_PARAM,
		&__aud_ctrl, FIELD_OFFSET(QPCODEC_AUDIO_CONTROL_PARAM, aou_pass_en),
		{
			{0, "0"}, {1, "1"},
			{0, 0}
		}
	},
	{
		"AudioControlParam-AiMsb", PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_PARAM,
		PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_PARAM,
		&__aud_ctrl, FIELD_OFFSET(QPCODEC_AUDIO_CONTROL_PARAM, ai_msb),
		{
			{0, "lsb"}, {1, "msb"},
			{0, 0}
		}
	},
	{
		"AudioControlParam-LrclkI", PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_PARAM,
		PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_PARAM,
		&__aud_ctrl, FIELD_OFFSET(QPCODEC_AUDIO_CONTROL_PARAM, lrclk_i),
		{
			{0, "high"}, {1, "low"},
			{0, 0}
		}
	},
	{
		"AudioControlParam-BclkI", PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_PARAM,
		PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_PARAM,
		&__aud_ctrl, FIELD_OFFSET(QPCODEC_AUDIO_CONTROL_PARAM, bclk_i),
		{
			{0, "rising"}, {1, "falling"},
			{0, 0}
		}
	},
	{
		"AudioControlParam-AiI2s", PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_PARAM,
		PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_PARAM,
		&__aud_ctrl, FIELD_OFFSET(QPCODEC_AUDIO_CONTROL_PARAM, ai_i2s),
		{
			{0, "normal"}, {1, "program"},
			{0, 0}
		}
	},
	{
		"AudioControlParam-AiRj", PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_PARAM,
		PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_PARAM,
		&__aud_ctrl, FIELD_OFFSET(QPCODEC_AUDIO_CONTROL_PARAM, ai_rj),
		{
			{0, "left"}, {1, "right"},
			{0, 0}
		}
	},
	{
		"AudioControlParam-AiM", PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_PARAM,
		PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_PARAM,
		&__aud_ctrl, FIELD_OFFSET(QPCODEC_AUDIO_CONTROL_PARAM, ai_m),
		{
			{0, "slave"}, {1, "master"},
			{0, 0}
		}
	},
	{
		"AudioControlParam-ChNum", PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_PARAM,
		PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_PARAM,
		&__aud_ctrl, FIELD_OFFSET(QPCODEC_AUDIO_CONTROL_PARAM, ch_num),
		{
			{0, "0"}, {1, "1"}, {2, "2"},
			{0, 0}
		}
	},
	{
		"AudioControlParam-Pts", PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_PARAM,
		PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_PARAM,
		&__aud_ctrl, FIELD_OFFSET(QPCODEC_AUDIO_CONTROL_PARAM, pts),
		{
			{0, 0}
		}
	},
	{
		"AudioControlParam-DisAi", PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_PARAM,
		PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_PARAM,
		&__aud_ctrl, FIELD_OFFSET(QPCODEC_AUDIO_CONTROL_PARAM, disAi),
		{
			{0, "0"}, {1, "1"},
			{0, 0}
		}
	},
	{
		"AudioControlParam-Raw", PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_PARAM,
		PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_PARAM,
		&__aud_ctrl, FIELD_OFFSET(QPCODEC_AUDIO_CONTROL_PARAM, raw),
		{
			{0, 0}
		}
	},
	{
		"AudioControlParam-AudioType", PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_PARAM,
		PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_PARAM,
		&__aud_ctrl, FIELD_OFFSET(QPCODEC_AUDIO_CONTROL_PARAM, audio_type),
		{
			{0, "lpcm"}, {1, "g726"}, {2, "aac"}, {3, "mp1"}, {4, "mp2"},
			{5, "mp3"}, {6, "ac3"}, {7, "g723"}, {8, "g711"}, {9, "raw"},
			{10, "g729"}, {11, "silk"},
			{0, 0}
		}
	},

    /* AudioControlEx */
	{
		"AudioControlEx-AacVer", PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_EX,
		PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_EX,
		&__aud_ctrl_ex, FIELD_OFFSET(QPCODEC_AUDIO_CONTROL_EX, aac_ver),
		{
			{2, "mpeg2"}, {4, "mpeg4"},
			{0, 0}
		}
	},
	{
		"AudioControlEx-HType", PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_EX,
		PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_EX,
		&__aud_ctrl_ex, FIELD_OFFSET(QPCODEC_AUDIO_CONTROL_EX, h_type),
		{
			{0, "adts"}, {1, "ga"},
			{0, 0}
		}
	},
	{
		"AudioControlEx-TNS", PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_EX,
		PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_EX,
		&__aud_ctrl_ex, FIELD_OFFSET(QPCODEC_AUDIO_CONTROL_EX, TNS),
		{
			{0, 0}
		}
	},
	{
		"AudioControlEx-IS", PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_EX,
		PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_EX,
		&__aud_ctrl_ex, FIELD_OFFSET(QPCODEC_AUDIO_CONTROL_EX, IS),
		{
			{0, 0}
		}
	},
	{
		"AudioControlEx-PNS", PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_EX,
		PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_EX,
		&__aud_ctrl_ex, FIELD_OFFSET(QPCODEC_AUDIO_CONTROL_EX, PNS),
		{
			{0, 0}
		}
	},
	{
		"AudioControlEx-MS", PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_EX,
		PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_EX,
		&__aud_ctrl_ex, FIELD_OFFSET(QPCODEC_AUDIO_CONTROL_EX, MS),
		{
			{0, 0}
		}
	},
	{
		"AudioControlEx-CutoffFreq", PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_EX,
		PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_EX,
		&__aud_ctrl_ex, FIELD_OFFSET(QPCODEC_AUDIO_CONTROL_EX, cutoff_freq),
		{
			{0, 0}
		}
	},
	{
		"AudioControlEx-Mode", PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_EX,
		PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_EX,
		&__aud_ctrl_ex, FIELD_OFFSET(QPCODEC_AUDIO_CONTROL_EX, mode),
		{
			{0, "a-law"}, {1, "u-law"},
			{0, 0}
		}
	},
	{
		"AudioControlEx-BlockSize", PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_EX,
		PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_EX,
		&__aud_ctrl_ex, FIELD_OFFSET(QPCODEC_AUDIO_CONTROL_EX, block_size),
		{
			{0, "80"}, {1, "160"}, {2, "240"}, {3, "320"},
			{0, 0}
		}
	},
	{
		"AudioControlEx-SampleCnt", PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_EX,
		PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_EX,
		&__aud_ctrl_ex, FIELD_OFFSET(QPCODEC_AUDIO_CONTROL_EX, sample_cnt),
		{
			{0, 0}
		}
	},
	{
		"AudioControlEx-RawType", PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_EX,
		PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_EX,
		&__aud_ctrl_ex, FIELD_OFFSET(QPCODEC_AUDIO_CONTROL_EX, raw_audio_type),
		{
			{0, "lpcm"}, {1, "ac3"}, {2, "aac"}, {3, "aac2"}, {4, "dts"},
			{0, 0}
		}
	},
	{
		"AudioControlEx-FrameLen", PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_EX,
		PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_EX,
		&__aud_ctrl_ex, FIELD_OFFSET(QPCODEC_AUDIO_CONTROL_EX, frame_len),
		{
			{0, "20"}, {1, "40"}, {2, "60"},
			{0, 0}
		}
	},
	{
		"AudioControlEx-DownSample", PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_EX,
		PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_EX,
		&__aud_ctrl_ex, FIELD_OFFSET(QPCODEC_AUDIO_CONTROL_EX, downsample),
		{
			{0, "0"}, {1, "1"},
			{0, 0}
		}
	},
	{
		"AudioControlEx-LRSwap", PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_EX,
		PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_EX,
		&__aud_ctrl_ex, FIELD_OFFSET(QPCODEC_AUDIO_CONTROL_EX, lr_swap),
		{
			{0, "0"}, {1, "1"},
			{0, 0}
		}
	},

    /* SysLink */
	{
		"SysLink-VideoInput", PLDEV_CNTL_IOCTL_SET_SYS_LINK,
		PLDEV_CNTL_IOCTL_GET_SYS_LINK,
		&__sys_link, FIELD_OFFSET(QPCODEC_SYS_LINK, video_input),
		{
			{0, "viu"}, {1, "host"}, {2, "usb"}, {4, "null"}, {8, "internal"},
			{0, 0}
		}
	},
	{
		"SysLink-VideoInCh", PLDEV_CNTL_IOCTL_SET_SYS_LINK,
		PLDEV_CNTL_IOCTL_GET_SYS_LINK,
		&__sys_link, FIELD_OFFSET(QPCODEC_SYS_LINK, video_in_ch),
		{
			{0, 0}
		}
	},
	{
		"SysLink-VideoOutput", PLDEV_CNTL_IOCTL_SET_SYS_LINK,
		PLDEV_CNTL_IOCTL_GET_SYS_LINK,
		&__sys_link, FIELD_OFFSET(QPCODEC_SYS_LINK, video_output),
		{
			{0, "vou"}, {1, "host"}, {2, "special"}, {4, "null"}, {8, "internal"},
			{0, 0}
		}
	},
	{
		"SysLink-VideoOutCh", PLDEV_CNTL_IOCTL_SET_SYS_LINK,
		PLDEV_CNTL_IOCTL_GET_SYS_LINK,
		&__sys_link, FIELD_OFFSET(QPCODEC_SYS_LINK, video_out_ch),
		{
			{0, 0}
		}
	},

	{
		"SysLink-AudioInput", PLDEV_CNTL_IOCTL_SET_SYS_LINK,
		PLDEV_CNTL_IOCTL_GET_SYS_LINK,
		&__sys_link, FIELD_OFFSET(QPCODEC_SYS_LINK, audio_input),
		{
			{0, "aiu"}, {1, "host"}, {8, "internal"},
			{0, 0}
		}
	},
	{
		"SysLink-AudioInCh", PLDEV_CNTL_IOCTL_SET_SYS_LINK,
		PLDEV_CNTL_IOCTL_GET_SYS_LINK,
		&__sys_link, FIELD_OFFSET(QPCODEC_SYS_LINK, audio_in_ch),
		{
			{0, 0}
		}
	},

	{
		"SysLink-AudioOutput", PLDEV_CNTL_IOCTL_SET_SYS_LINK,
		PLDEV_CNTL_IOCTL_GET_SYS_LINK,
		&__sys_link, FIELD_OFFSET(QPCODEC_SYS_LINK, audio_output),
		{
			{0, "aou"}, {1, "host"}, {8, "internal"},
			{0, 0}
		}
	},

	{
		"SysLink-AudioOutCh", PLDEV_CNTL_IOCTL_SET_SYS_LINK,
		PLDEV_CNTL_IOCTL_GET_SYS_LINK,
		&__sys_link, FIELD_OFFSET(QPCODEC_SYS_LINK, audio_out_ch),
		{
			{0, 0}
		}
	},

    /* SysFunction */
	{
		"SysFunction-Function", PLDEV_CNTL_IOCTL_SET_SYS_FUNCTION,
		PLDEV_CNTL_IOCTL_GET_SYS_FUNCTION,
		&__sys_func, FIELD_OFFSET(QPCODEC_SYS_FUNCTION, function),
		{
			{0, "encode"}, {1, "decode"},
			{0, 0}
		}
	},

	{
		"SysFunction-Video", PLDEV_CNTL_IOCTL_SET_SYS_FUNCTION,
		PLDEV_CNTL_IOCTL_GET_SYS_FUNCTION,
		&__sys_func, FIELD_OFFSET(QPCODEC_SYS_FUNCTION, function),
		{
			{0, "none"}, {1, "h264"}, {2, "jpeg"}, {4, "passthrough"},
			{5, "decimation"}, {6, "motiondetect"}, {7, "viu_osd"},
			{8, "vou_osd"},
			{0, 0}
		}
	},
	{
		"SysFunction-Audio", PLDEV_CNTL_IOCTL_SET_SYS_FUNCTION,
		PLDEV_CNTL_IOCTL_GET_SYS_FUNCTION,
		&__sys_func, FIELD_OFFSET(QPCODEC_SYS_FUNCTION, function),
		{
			{0, "none"}, {1, "audio"}, {4, "passthrough"},
			{0, 0}
		}
	},
	{
		"DeInterlace-Mode", PLDEV_CNTL_IOCTL_SET_DEINTERLACE,
		PLDEV_CNTL_IOCTL_GET_DEINTERLACE,
		&__sys_func, 0,
		{
			{0, "none"}, {1, "topfield"}, {2, "twofield"},
			{0, 0}
		}
	},
	{
		NULL, 0, 0, NULL, 0, { {0, 0} }
	},
};

static EQP_Errors __pl330_set_config(CComponent *component, unsigned int cmd, void *arg)
{
	unsigned int nr = _IOC_NR(cmd);
	EQP_Errors qpStatus = QPERR_PARMS;
	IMpegCodec *pMpegCodec = CDevice_getMpegCodec(component->m_pDevice);

	/* call codec/cqlcodec.c CQLCodec_Set() */
	qpStatus = pMpegCodec->Set(pMpegCodec,
							   (TQP_REFGUID)&PROPSETID_QPCODEC_CONTROL,
							   component->m_hTask,
							   nr - PLDEV_CNTL_IOCTL_SET,
							   (TQP_PVOID)arg,
							   QP_NULL, 0);
	if (!QP_SUCCESS(qpStatus))
	{
	}

	return QPERR_SUCCESS;
}

static EQP_Errors __pl330_get_config(CComponent *component, unsigned int cmd, void *arg)
{
	unsigned int nr = _IOC_NR(cmd);
	EQP_Errors qpStatus = QPERR_PARMS;
	TQP_UINT32	dwGot = 0;
	IMpegCodec *pMpegCodec = CDevice_getMpegCodec(component->m_pDevice);

	/* get current settigs */
	qpStatus = pMpegCodec->Get(pMpegCodec,
			(TQP_REFGUID)&PROPSETID_QPCODEC_CONTROL,
			component->m_hTask,
			nr - PLDEV_CNTL_IOCTL_GET,
			(TQP_PVOID)arg, (TQP_PVOID)arg, &dwGot);

	if (!QP_SUCCESS(qpStatus))
	{
	}

	return QPERR_SUCCESS;
}

/* fill property offset is zero */
static void fill_bits_qpcodec_ctrl(char *val, TQP_INT32 set_val, bool *handle)
{
	if(!strcmp(val, "SystemControl-Version"))
		__sys_ctrl.Artesa.version = set_val;
	else if(!strcmp(val, "SystemControl-XferMode"))
		__sys_ctrl.Artesa.xfer_mode = set_val;
	else if(!strcmp(val, "InputControl-InFrameRate"))
		__input_ctrl.Artesa.in_frame_rate = (set_val | IN_FRAME_RATE_ABSOLUTE);
	else if(!strcmp(val, "SysFunction-Function"))
		__sys_func.functionEx.function = set_val;
	else if(!strcmp(val, "SysFunction-Video"))
		__sys_func.functionEx.video = set_val;
	else if(!strcmp(val, "SysFunction-Audio"))
		__sys_func.functionEx.audio = set_val;
	else
		*handle = false;
}

/* parse setting value */
TQP_INT32 parse_item(struct qpcodec_item_config_table *table_item,
		void *val,
		bool *handle, bool decimal)
{
	long set_val = 0;
	int index = 0;
	char *str_val;
	struct config_item *item = table_item->item;
	TQP_INT32 decimal_val;

	if(decimal)
		decimal_val = *((TQP_INT32 *) val);
	else
		str_val = (char *) val;

	while(item[index].desc != NULL)
	{
		if(!strcmp(val, item[index].desc))
			break;
		index++;
	}


	if(item[0].desc != NULL)
	{
		set_val = item[index].val;
		if(!strcmp(table_item->item_name, "SysFunction-Function"))
			fill_bits_qpcodec_ctrl(table_item->item_name, set_val, handle);
		else if(!strcmp(table_item->item_name, "SysFunction-Video"))
			fill_bits_qpcodec_ctrl(table_item->item_name, set_val, handle);
		else if(!strcmp(table_item->item_name, "SysFunction-Audio"))
			fill_bits_qpcodec_ctrl(table_item->item_name, set_val, handle);
		else
			return set_val;
	}
	else
	{
		if(decimal)
		{
			decimal_val = *((TQP_INT32 *) val);
			set_val = decimal_val;
			return set_val;
		}
		else
			kstrtol(val, 10, &set_val);
		*handle = true;
		fill_bits_qpcodec_ctrl(table_item->item_name, set_val, handle);
		return set_val;
	}

	*handle = true;

	return set_val;
}

void parse_table(CComponent *component, char *item, void *val,
		unsigned int *i, bool decimal)
{
	int index;
	bool handle = false;
	struct qpcodec_item_config_table *table_item;
	int set_val = 0;


	for(index = 0; pl330_config_table[index].item_name != NULL; index++)
	{
		if(!strcmp(item, pl330_config_table[index].item_name))
		{
			handle = false;
			table_item = &pl330_config_table[index];

			__pl330_get_config(component,
					table_item->get_ctrl_id,
					table_item->qpcodec_ctrl);

			set_val = parse_item(table_item, val, &handle, decimal);

			if(!handle)
				memcpy(table_item->qpcodec_ctrl + table_item->offset,
						&set_val,
						sizeof(TQP_INT32));

			pl330_trace(PL330_TRACE_STATS, "%s item:%s val:%d\n", __func__, item, set_val);
			*i = index;
			break;
		}
	}
}

int pl330_set_config(CComponent *component, char *item, void *val, bool decimal)
{
	unsigned int index = -1;
	EQP_Errors status;

	if(!component)
		return -EIO;

	parse_table(component, item, val, &index, decimal);
	
	if(index == -1)
	{
		if(decimal)
			printk("%s error invalid input %s:%d\n", __func__, item, *((int *)val));
		else
			printk("%s error invalid input %s:%s\n", __func__, item, (char *)val);
		return -EINVAL;
	}

	status = __pl330_set_config(component,
				pl330_config_table[index].set_ctrl_id,
				pl330_config_table[index].qpcodec_ctrl);

	return status;
}


int pl330_port_open(CComponent *component, PCOMPONENT_PORT_OPEN open_args)
{
	EQP_Errors status;

	status = ccomponent_port_open(component, open_args);

	if(status != QPERR_SUCCESS)
		pl330_trace(PL330_TRACE_PORT,"%s() error\n", __func__);
	return status;
}

int pl330_port_close(CComponent *component, COMPONENT_PORT_TYPE type)
{
	EQP_Errors status;

	status = ccomponent_port_close(component, type);
	return status;
}

int pl330_acquire_index(CComponent *component, COMPONENT_PORT_TYPE type)
{
	return ccomponent_acquire_index(component, type);
}

int pl330_acquire(CComponent *component)
{
	return ccomponent_acquire(component);
}
