#ifndef __PL330_H__
#define __PL330_H__

#include "cdevice.h"
#include "ccomponent.h"
#include "qpmpgcodec.h"
#include "cqlcodeclib.h"

#include <linux/videodev2.h>
#include <sound/pcm.h>

#define FRAMEINVERVAL(x) \
	(x ==  FRAME_INTERVAL_15) ? 15 : \
	(x ==  FRAME_INTERVAL_24) ? 24 : \
	(x ==  FRAME_INTERVAL_25) ? 25 : \
	(x ==  FRAME_INTERVAL_30) ? 30 : \
	(x ==  FRAME_INTERVAL_50) ? 50 : \
	(x ==  FRAME_INTERVAL_60) ? 60 : \
	(x ==  FRAME_INTERVAL_70) ? 70 : \
	(x ==  FRAME_INTERVAL_72) ? 72 : \
	(x ==  FRAME_INTERVAL_75) ? 75 : \
	(x ==  FRAME_INTERVAL_85) ? 85 : 0

enum frameinterval
{
	FRAME_INTERVAL_15,
	FRAME_INTERVAL_24,
	FRAME_INTERVAL_25,
	FRAME_INTERVAL_30,
	FRAME_INTERVAL_50,
	FRAME_INTERVAL_60,
	FRAME_INTERVAL_70,
	FRAME_INTERVAL_72,
	FRAME_INTERVAL_75,
	FRAME_INTERVAL_85,
};

struct pl330_audio_dev
{
	CDevice			*dev;
	CComponent		*component;
	CPort			*port;

	struct pci_dev	*pci;

	COMPONENT_PORT_TYPE		port_type;
	QPMPGCODEC_OPEN_TYPE	mpg_codec;

	spinlock_t		lock;

	struct snd_card	*card;
	struct snd_pcm_substream *substream;
	struct work_struct wq_trigger;
	atomic_t		stream_started;


	struct v4l2_buffer	*buf;
	snd_pcm_uframes_t capture_transfer_done;
	snd_pcm_uframes_t hwptr;

	bool running;

	void (*on_buffer_done)(CComponent *component);
};

struct pl330_frame
{
	u16 width;
	u16 height;
	u32 min_bit_rate;
	u32 max_bit_rate;
	u32 avg_bit_rate;

	u32 frame_rate;

	u32	frame_type;
	const unsigned int *frameinterval;

	u32	analog_frame_type;
	const unsigned int *analog_frameinterval;
};

struct pl330_format
{
	char *name;
	u32 pixelformat;
	u8 depth;
	u8 flags;

	struct pl330_frame frame;
};

int pl330_set_config(CComponent *component, char *item, void *val, bool decimal);

int pl330_port_open(CComponent *component, PCOMPONENT_PORT_OPEN open_args);

int pl330_port_close(CComponent *component, COMPONENT_PORT_TYPE type);

int pl330_acquire_index(CComponent *component, COMPONENT_PORT_TYPE type);

int pl330_acquire(CComponent *component);

#endif
