#include "tlv320aic3101.h"
#include "tlv320aic3x.h"

#include "qptimemgr.h"
#include "qpdbgmgr.h"
#include "util.h"
#include "cqlcodeclib.h"
#include "ci2c.h"

#include <sound/pcm.h>



static __inline void udelay_loop(TQPTM_Time uS)
{
	while(uS--)
	{
		asm volatile("nop;"); asm volatile("nop;");	asm volatile("nop;"); asm volatile("nop;");
		asm volatile("nop;"); asm volatile("nop;");	asm volatile("nop;"); asm volatile("nop;");
		asm volatile("nop;"); asm volatile("nop;");	asm volatile("nop;"); asm volatile("nop;");
		asm volatile("nop;"); asm volatile("nop;");	asm volatile("nop;"); asm volatile("nop;");
		asm volatile("nop;"); asm volatile("nop;");	asm volatile("nop;"); asm volatile("nop;");
		asm volatile("nop;"); asm volatile("nop;");	asm volatile("nop;"); asm volatile("nop;");
		asm volatile("nop;"); asm volatile("nop;");	asm volatile("nop;"); asm volatile("nop;");
		asm volatile("nop;"); asm volatile("nop;");	asm volatile("nop;"); asm volatile("nop;");
		asm volatile("nop;"); asm volatile("nop;");	asm volatile("nop;"); asm volatile("nop;");
		asm volatile("nop;"); asm volatile("nop;");	asm volatile("nop;"); asm volatile("nop;");
		asm volatile("nop;"); asm volatile("nop;");	asm volatile("nop;"); asm volatile("nop;");
		asm volatile("nop;"); asm volatile("nop;");	asm volatile("nop;"); asm volatile("nop;");
		asm volatile("nop;"); asm volatile("nop;");	asm volatile("nop;"); asm volatile("nop;");
		asm volatile("nop;"); asm volatile("nop;");	asm volatile("nop;"); asm volatile("nop;");
		asm volatile("nop;"); asm volatile("nop;");	asm volatile("nop;"); asm volatile("nop;");
		asm volatile("nop;"); asm volatile("nop;");	asm volatile("nop;"); asm volatile("nop;");
		asm volatile("nop;"); asm volatile("nop;");	asm volatile("nop;"); asm volatile("nop;");
		asm volatile("nop;"); asm volatile("nop;");	asm volatile("nop;"); asm volatile("nop;");
		asm volatile("nop;"); asm volatile("nop;");	asm volatile("nop;"); asm volatile("nop;");
		asm volatile("nop;"); asm volatile("nop;");	asm volatile("nop;"); asm volatile("nop;");
		asm volatile("nop;"); asm volatile("nop;");	asm volatile("nop;"); asm volatile("nop;");
		asm volatile("nop;"); asm volatile("nop;");	asm volatile("nop;"); asm volatile("nop;");
		asm volatile("nop;"); asm volatile("nop;");	asm volatile("nop;"); asm volatile("nop;");
		asm volatile("nop;"); asm volatile("nop;");	asm volatile("nop;"); asm volatile("nop;");
		asm volatile("nop;"); asm volatile("nop;");	asm volatile("nop;"); asm volatile("nop;");
		asm volatile("nop;"); asm volatile("nop;");	asm volatile("nop;"); asm volatile("nop;");
	}
};

static unsigned int aic3101_i2c_write(CAIC3101 *This,
										unsigned int reg, unsigned val)
{
    TQP_UINT8 buffer[2];

	buffer[0] = reg;
	buffer[1] = val;

	return (II2C_Write(This->m_pI2C, 
					   This->m_cI2CDevAddr,
					   2,					//write size
					   buffer				//write buffer
					   ));
}

static unsigned int aic3101_i2c_read(CAIC3101 *This,
										unsigned int reg)
{
	TQP_UINT8 value = 0;
	EQP_Errors err;
        err = II2C_WriteThenRead(This->m_pI2C, This->m_cI2CDevAddr, 1,	(PTQP_UINT8)&reg, 1, &value);
	if(err != QPERR_SUCCESS)
	{
		printk("%s Device:%x reg:%x, value:%x error %d\n",
				__func__, This->m_cI2CDevAddr, reg, value, err);
		return -EIO;
	}
	return value;
}

static  __attribute__((__unused__)) unsigned int aic3101_update_bits(CAIC3101 *This, unsigned short reg,
											unsigned int mask, unsigned int val)
{
	bool change;
	unsigned int new, old;
	int ret;

	ret = aic3101_i2c_read(This, reg);
	if(ret < 0)
		return ret;

	old = ret;
	new = (old & ~mask) | (val & mask);
	change = old != new;

	if(change)
		ret = aic3101_i2c_write(This, reg, new);

	if(ret < 0)
		return ret;

	return change;
}

static int aic3101_init(CAIC3101 *This)
{
	struct aic3101_priv *priv = &This->priv;
	u8 data;
	int delay = 0;

	aic3101_i2c_write(This, AIC3X_PAGE_SELECT, PAGE0_SELECT);
//	aic3101_i2c_write(This, AIC3X_RESET, SOFT_RESET);

	/* enable PLL Q=17, P=1, J=2 */
	aic3101_i2c_write(This, AIC3X_PLL_PROGA_REG, 0x91); 
	aic3101_i2c_write(This, AIC3X_PLL_PROGB_REG, 0x20); 

	/* set BCLK, WCLK master/slave mode */
	aic3101_i2c_write(This, AIC3X_ASD_INTF_CTRLA,
			priv->master ?
			(BIT_CLK_MASTER | WORD_CLK_MASTER) :
			~(BIT_CLK_MASTER | WORD_CLK_MASTER));

	/* select data word length */
	data = aic3101_i2c_read(This, AIC3X_ASD_INTF_CTRLB) & 0x0f;
	switch (priv->word_len) {
	case SNDRV_PCM_FORMAT_S16_LE:
		break;
	case SNDRV_PCM_FORMAT_S20_3LE:
		data |= (0x01 << 4);
		break;
	case SNDRV_PCM_FORMAT_S24_LE:
		data |= (0x02 << 4);
		break;
	case SNDRV_PCM_FORMAT_S32_LE:
		data |= (0x03 << 4);
		break;
	}

	/* select mode */
	switch (priv->dai_fmt) {
	case (SND_SOC_DAIFMT_I2S | SND_SOC_DAIFMT_NB_NF):
		break;
	case (SND_SOC_DAIFMT_DSP_A | SND_SOC_DAIFMT_IB_NF):
		delay = 1;
	case (SND_SOC_DAIFMT_DSP_B | SND_SOC_DAIFMT_IB_NF):
		data |= (0x01 << 6);
		break;
	case (SND_SOC_DAIFMT_RIGHT_J | SND_SOC_DAIFMT_NB_NF):
		data |= (0x02 << 6);
		break;
	case (SND_SOC_DAIFMT_LEFT_J | SND_SOC_DAIFMT_NB_NF):
		data |= (0x03 << 6);
		break;
	}
	aic3101_i2c_write(This, AIC3X_ASD_INTF_CTRLB, data);
	aic3101_i2c_write(This, AIC3X_ASD_INTF_CTRLC, delay);


	/* LR ADC HP Filter 0.0045xFs */
	aic3101_i2c_write(This, AIC3X_CODEC_DFILT_CTRL, 0x50);
#if 1
	/* DAC default volume and mute */
	aic3101_i2c_write(This, LDAC_VOL, DEFAULT_VOL | MUTE_ON);
	aic3101_i2c_write(This, RDAC_VOL, DEFAULT_VOL | MUTE_ON);
#endif

#if 0
	/* DAC to HP default volume and route to Output mixer */
	aic3101_i2c_write(This, DACL1_2_HPLOUT_VOL, DEFAULT_VOL | ROUTE_ON);
	aic3101_i2c_write(This, DACR1_2_HPROUT_VOL, DEFAULT_VOL | ROUTE_ON);
	aic3101_i2c_write(This, DACL1_2_HPLCOM_VOL, DEFAULT_VOL | ROUTE_ON);
	aic3101_i2c_write(This, DACR1_2_HPRCOM_VOL, DEFAULT_VOL | ROUTE_ON);
	/* DAC to Line Out default volume and route to Output mixer */
	aic3101_i2c_write(This, DACL1_2_LLOPM_VOL, DEFAULT_VOL | ROUTE_ON);
	aic3101_i2c_write(This, DACR1_2_RLOPM_VOL, DEFAULT_VOL | ROUTE_ON);
	/* DAC to Mono Line Out default volume and route to Output mixer */
	aic3101_i2c_write(This, DACL1_2_MONOLOPM_VOL, DEFAULT_VOL | ROUTE_ON);
	aic3101_i2c_write(This, DACR1_2_MONOLOPM_VOL, DEFAULT_VOL | ROUTE_ON);

	/* unmute all outputs */
	aic3101_update_bits(This, LLOPM_CTRL, UNMUTE, UNMUTE);
	aic3101_update_bits(This, RLOPM_CTRL, UNMUTE, UNMUTE);
	aic3101_update_bits(This, MONOLOPM_CTRL, UNMUTE, UNMUTE);
	aic3101_update_bits(This, HPLOUT_CTRL, UNMUTE, UNMUTE);
	aic3101_update_bits(This, HPROUT_CTRL, UNMUTE, UNMUTE);
	aic3101_update_bits(This, HPLCOM_CTRL, UNMUTE, UNMUTE);
	aic3101_update_bits(This, HPRCOM_CTRL, UNMUTE, UNMUTE);
#endif

	/* ADC default volume and unmute */
#if 0
	aic3101_i2c_write(This, LADC_VOL, DEFAULT_GAIN);
	aic3101_i2c_write(This, RADC_VOL, DEFAULT_GAIN);
#else
	aic3101_i2c_write(This, LADC_VOL, 0x0e);
	aic3101_i2c_write(This, RADC_VOL, 0x0e);
#endif

#if 0
	/* By default route Line1 to ADC PGA mixer */
	aic3101_i2c_write(This, LINE1L_2_LADC_CTRL, 0x0);
	aic3101_i2c_write(This, LINE1R_2_RADC_CTRL, 0x0);
#else
	/* Powered up left/right ADC channel */
	aic3101_i2c_write(This, LINE1L_2_LADC_CTRL, 0x04);
	aic3101_i2c_write(This, LINE1R_2_RADC_CTRL, 0x04);
#endif

	/* LINE1 L/R is not connected to the right/left-ADC PGA */
	aic3101_i2c_write(This, LINE1R_2_LADC_CTRL, 0x78);
	aic3101_i2c_write(This, LINE1L_2_RADC_CTRL, 0x78);

#if 0
	/* PGA to HP Bypass default volume, disconnect from Output Mixer */
	aic3101_i2c_write(This, PGAL_2_HPLOUT_VOL, DEFAULT_VOL);
	aic3101_i2c_write(This, PGAR_2_HPROUT_VOL, DEFAULT_VOL);
	aic3101_i2c_write(This, PGAL_2_HPLCOM_VOL, DEFAULT_VOL);
	aic3101_i2c_write(This, PGAR_2_HPRCOM_VOL, DEFAULT_VOL);
	/* PGA to Line Out default volume, disconnect from Output Mixer */
	aic3101_i2c_write(This, PGAL_2_LLOPM_VOL, DEFAULT_VOL);
	aic3101_i2c_write(This, PGAR_2_RLOPM_VOL, DEFAULT_VOL);
	/* PGA to Mono Line Out default volume, disconnect from Output Mixer */
	aic3101_i2c_write(This, PGAL_2_MONOLOPM_VOL, DEFAULT_VOL);
	aic3101_i2c_write(This, PGAR_2_MONOLOPM_VOL, DEFAULT_VOL);

	/* Line2 to HP Bypass default volume, disconnect from Output Mixer */
	aic3101_i2c_write(This, LINE2L_2_HPLOUT_VOL, DEFAULT_VOL);
	aic3101_i2c_write(This, LINE2R_2_HPROUT_VOL, DEFAULT_VOL);
	aic3101_i2c_write(This, LINE2L_2_HPLCOM_VOL, DEFAULT_VOL);
	aic3101_i2c_write(This, LINE2R_2_HPRCOM_VOL, DEFAULT_VOL);
	/* Line2 Line Out default volume, disconnect from Output Mixer */
	aic3101_i2c_write(This, LINE2L_2_LLOPM_VOL, DEFAULT_VOL);
	aic3101_i2c_write(This, LINE2R_2_RLOPM_VOL, DEFAULT_VOL);
	/* Line2 to Mono Out default volume, disconnect from Output Mixer */
	aic3101_i2c_write(This, LINE2L_2_MONOLOPM_VOL, DEFAULT_VOL);
	aic3101_i2c_write(This, LINE2R_2_MONOLOPM_VOL, DEFAULT_VOL);
#endif
        return 0;
}
#if 0
struct aic3101_priv *aic3101_attach(const struct aic3101_config *config,
					II2C *i2c)
{
	struct aic3101_priv *priv = NULL;
	int status;

	priv = kzalloc(sizeof(struct aic3101_priv), GFP_KERNEL);

	if(!priv)
		return NULL;

	priv->adr = config->adr;
	priv->i2c = i2c;
	priv->gpio_reset = config->gpio_reset;
/*
	write_gpio(&pQLCodec->m_hci, 0, 0);
	udelay_loop(5);
	write_gpio(&pQLCodec->m_hci, 0, 1);
	udelay_loop(5);
*/

	aic3101_init(priv);

	return priv;
}
#endif

int aic3101_mute(CAIC3101 *This, bool mute)
{
	u8 ladc_reg = aic3101_i2c_read(This, LADC_VOL) & ~MUTE_ON;
	u8 radc_reg = aic3101_i2c_read(This, RADC_VOL) & ~MUTE_ON;

	if (mute) {
		aic3101_i2c_write(This, LADC_VOL, ladc_reg | MUTE_ON);
		aic3101_i2c_write(This, RADC_VOL, radc_reg | MUTE_ON);
	} else {
		aic3101_i2c_write(This, LADC_VOL, ladc_reg);
		aic3101_i2c_write(This, RADC_VOL, radc_reg);
	}

	return 0;
}

bool aic3101_is_mute(CAIC3101 *This)
{
	u8 ladc_reg = aic3101_i2c_read(This, LADC_VOL) & ~MUTE_ON;
	u8 radc_reg = aic3101_i2c_read(This, RADC_VOL) & ~MUTE_ON;

	if((ladc_reg & MUTE_ON == MUTE_ON) &&
	   (radc_reg & MUTE_ON == MUTE_ON))
		return true;
	return false;
}

// IGenericDevice
//
static EQP_Errors CAIC3101_InitDevice(IAudioCodec *This_p,
									 QP_DEVICE_CALLBACK pFuncCallback,
									 TQP_PVOID context,
									 TQP_PVOID pInitData
									 )
{
    CAudioCodec *pAudioCodec = GET_INHERITED_OBJECT(CAudioCodec, m_iAudCodec, This_p);
    CAIC3101      *This = GET_INHERITED_OBJECT(CAIC3101, m_AudioCodec, pAudioCodec);
	struct aic3101_priv *priv = &This->priv;

	CQLCodecLib	*codeclib = GET_INHERITED_OBJECT(CQLCodecLib, m_Object, pAudioCodec->m_Object.m_pParent);
	CQLCodec *pQLCodec = codeclib->m_pMpgCodec;

    //    CI2C *pCI2C = GET_INHERITED_OBJECT(CI2C, m_ii2c, This->m_pI2C);
	//EQP_Errors	qpStatus=QPERR_SUCCESS;

    QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
                 "CAIC3101_InitDevice()\n"
                ));

    if (CObject_IsInitialized(&pAudioCodec->m_Object))
    {
        QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
                     "CAIC3101_InitDevice() Already Initialized. Skipping...\n"
                    ));
      return (QPERR_SUCCESS);
    }

	priv->gpio_reset = AIC3101_GPIO_RESET;
	priv->master = 1;
	priv->word_len = SNDRV_PCM_FORMAT_S24_LE;
	priv->dai_fmt = SND_SOC_DAIFMT_I2S | SND_SOC_DAIFMT_NB_NF;

	/* reset AIC3101 */
	write_gpio(&pQLCodec->m_hci, priv->gpio_reset, 0);
	udelay_loop(5);
	write_gpio(&pQLCodec->m_hci, priv->gpio_reset, 1);
	udelay_loop(5);

	/* init AIC3101 */
	aic3101_init(This);

    // save callback address and context
    //
    pAudioCodec->m_pDeviceCallback = pFuncCallback;
    pAudioCodec->m_callbackContext = context;

    // CObject Init
    //
    CObject_Init(&pAudioCodec->m_Object);

    return (QPERR_SUCCESS);
}
EQP_Errors CAIC3101_Create(CQLCodecLib* This ,Chip_Cfg * chip_cfg)
{
    CAudioCodec		*pAudCodec;

    pAudCodec = (CAudioCodec *)QPMMMalloc2Ex(sizeof(CAIC3101));

    if (!pAudCodec)
    {
        QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
                    "CQLCodecLib_CreatePeripherals() pAudCodec == NULL!!!\n"
                    ));
        return (QPERR_FAIL);
    }

    This->m_pAudCodec = (CAudioCodec *) CAIC3101_Constructor((CAIC3101 *)pAudCodec,
            &This->m_Object,
            OBJECT_DEFAULT,
            I_I2C(This),
            chip_cfg->I2C_add_cfg 
            );

    if (QP_NULL == This->m_pAudCodec)
    {
        QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
                    "CQLCodecLib_CreatePeripherals() CFM31_Constructor() failed\n"
                    ));
        QPMMFree2Ex(pAudCodec);
        return (QPERR_FAIL);
    }
    return QPERR_SUCCESS;
}
void CAIC3101_Delete(CQLCodecLib* This)
{
    CAIC3101_Destructor((CAIC3101 *)GET_INHERITED_OBJECT(CAIC3101, m_AudioCodec, This));
}

CAIC3101 *CAIC3101_Constructor(CAIC3101 *pAIC3x,
								CObject *pParent,
								TQP_UINT32 dwAttributes,
								II2C *pI2C,
								TQP_UINT8 cI2CDevAddr)
{
	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CAIC3101_Constructor()\n"
				 ));

	if (pAIC3x)
	{
		CAudioCodec	*pAudioCodec;

		pAudioCodec = CAudioCodec_Constructor(&pAIC3x->m_AudioCodec,
												  pParent,
												  dwAttributes
												  );

		if (!pAudioCodec)
		{
			pAIC3x = QP_NULL;
		}
		else
		{
			// fill in the function table
			//

			// IGenericDevice
			pAIC3x->m_AudioCodec.m_iAudCodec.InitDevice = CAIC3101_InitDevice;

			// initialize members
			//
			pAIC3x->m_pI2C = pI2C;
			pAIC3x->m_cI2CDevAddr = cI2CDevAddr;
		}
	}

	return (pAIC3x);
}


void CAIC3101_Destructor(CAIC3101 *This)
{

    QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CAIC3101_Destructor()\n"
				 ));

    CAudioCodec_Destructor(&This->m_AudioCodec);
}
