#ifndef __TI3101_H__
#define __TI3101_H__

#include "caudcodec.h"
#include "cqlcodeclib.h"
#include "plgpio.h"
#include "boardinfo-probe.h"

#define AIC3101_GPIO_RESET 1
#define AIC3101_DEVICE_ADDR (0x30 >> 1)

/* kernel 3.7 */
/*
 * DAI hardware audio formats.
 *
 * Describes the physical PCM data formating and clocking. Add new formats
 * to the end.
 */
#define SND_SOC_DAIFMT_I2S		1 /* I2S mode */
#define SND_SOC_DAIFMT_RIGHT_J		2 /* Right Justified mode */
#define SND_SOC_DAIFMT_LEFT_J		3 /* Left Justified mode */
#define SND_SOC_DAIFMT_DSP_A		4 /* L data MSB after FRM LRC */
#define SND_SOC_DAIFMT_DSP_B		5 /* L data MSB during FRM LRC */
#define SND_SOC_DAIFMT_AC97		6 /* AC97 */
#define SND_SOC_DAIFMT_PDM		7 /* Pulse density modulation */

/*
 * DAI hardware signal inversions.
 *
 * Specifies whether the DAI can also support inverted clocks for the specified
 * format.
 */
#define SND_SOC_DAIFMT_NB_NF		(1 << 8) /* normal bit clock + frame */
#define SND_SOC_DAIFMT_NB_IF		(2 << 8) /* normal BCLK + inv FRM */
#define SND_SOC_DAIFMT_IB_NF		(3 << 8) /* invert BCLK + nor FRM */
#define SND_SOC_DAIFMT_IB_IF		(4 << 8) /* invert BCLK + FRM */

struct aic3101_config
{
	u8		adr;
	int		gpio_reset;
};

struct aic3101_priv
{
	int		gpio_reset;
	int		master;
	int		word_len;
	int		dai_fmt;
};

typedef struct CAIC3101
{
	// public CAudioCodec
	CAudioCodec				m_AudioCodec;

	II2C					*m_pI2C;				// I2C interface
	TQP_UINT8				m_cI2CDevAddr;			// i2c device address

	struct aic3101_priv		priv;

} CAIC3101;

CAIC3101 *CAIC3101_Constructor(CAIC3101 *pAIC3x,
								CObject *pParent,
								TQP_UINT32 dwAttributes,
								II2C *pI2C,
								TQP_UINT8 cI2CDevAddr);

void CAIC3101_Destructor(CAIC3101 *This);
EQP_Errors CAIC3101_Create(CQLCodecLib* This ,Chip_Cfg * chip_cfg);
void CAIC3101_Delete(CQLCodecLib* This);

int aic3101_mute(CAIC3101 *This, bool mute);

struct aic3101_priv *aic3101_attach(const struct aic3101_config *config,
						II2C *i2c);

bool aic3101_is_mute(CAIC3101 *This);

#endif
